@@ -39,11 +39,11 @@ import Darcs.Patch.Witnesses.Eq ( MyEq, unsafeCompare )
 import Darcs.Patch.Witnesses.Show
 import Darcs.Patch.Prim( PrimPatch, coalesce, FromPrim, PrimOf, PrimPatchBase )
 import qualified Darcs.Patch.Prim.V1 as V1 ( Prim )
-import qualified Darcs.Patch.Prim.V3 as V3 ( Prim )
+import qualified Darcs.Patch.Prim.FileUUID as FileUUID ( Prim )
 import Darcs.Patch.RepoPatch ( RepoPatch )
 import Darcs.Patch.Type ( PatchType(..) )
-import Darcs.Patch.V1 as V1 ( Patch )
-import Darcs.Patch.V2.Real ( isConsistent, isForward, RealPatch )
+import Darcs.Patch.V1 as V1 ( RepoPatchV1 )
+import Darcs.Patch.V2.RepoPatch ( isConsistent, isForward, RepoPatchV2 )
 import Darcs.Patch.Patchy ( Commute(..), Patchy )
 import Darcs.Patch.Merge( Merge )
 import Darcs.Patch.Show ( ShowPatchBasic )
@@ -51,14 +51,15 @@ import Darcs.Patch.Apply( ApplyState )
 
 import Darcs.Test.Patch.Arbitrary.Generic
 import qualified Darcs.Test.Patch.Arbitrary.PrimV1 as P1
-import Darcs.Test.Patch.Arbitrary.PrimV3()
-import Darcs.Test.Patch.Arbitrary.Real
-import Darcs.Test.Patch.Arbitrary.PatchV1 ()
+import Darcs.Test.Patch.Arbitrary.PrimFileUUID()
+import Darcs.Test.Patch.Arbitrary.RepoPatchV1 ()
+import Darcs.Test.Patch.Arbitrary.RepoPatchV2
 import Darcs.Test.Patch.Arbitrary.PrimV1 ()
 import Darcs.Test.Patch.RepoModel
 import Darcs.Test.Patch.WithState( WithState, WithStartState )
 
 import qualified Darcs.Test.Patch.Info
+import qualified Darcs.Test.Patch.Selection
 
 import qualified Darcs.Test.Patch.Examples.Set1 as Ex
 import qualified Darcs.Test.Patch.Examples.Set2Unwitnessed as ExU
@@ -67,7 +68,7 @@ import Darcs.Test.Patch.Properties.Check( Check(..) )
 import qualified Darcs.Test.Patch.Properties.V1Set1 as Prop1
 import qualified Darcs.Test.Patch.Properties.V1Set2 as Prop2
 import qualified Darcs.Test.Patch.Properties.Generic as PropG
-import qualified Darcs.Test.Patch.Properties.Real as PropR
+import qualified Darcs.Test.Patch.Properties.RepoPatchV2 as PropR
 import qualified Darcs.Test.Patch.Properties.GenericUnwitnessed as PropU
 
 import qualified Darcs.Test.Patch.Rebase as Rebase
@@ -124,29 +125,29 @@ unit_V2P1 =
   , testCases "FL prim inverses commute" (PropU.commuteInverses WSub.commute) $ ExU.commutablesFL
   , testCases "fails" (PropU.commuteFails WSub.commute) ([] :: [(V1.Prim WSub.:> V1.Prim) wX wY])
   , testCases "read and show work on Prim" PropU.show_read ExU.primPatches
-  , testCases "read and show work on RealPatch" PropU.show_read ExU.realPatches
-  , testCases "example flattenings work" PropU.consistentTreeFlattenings ExU.realPatchLoopExamples
-  , testCases "real merge input consistent" (PropU.mergeArgumentsConsistent isConsistent) ExU.realMergeables
-  , testCases "real merge input is forward" (PropU.mergeArgumentsConsistent isForward) ExU.realMergeables
-  , testCases "real merge output is forward" (PropU.mergeConsistent isForward) ExU.realMergeables
-  , testCases "real merge output consistent" (PropU.mergeConsistent isConsistent) ExU.realMergeables
-  , testCases "real merge either way" PropU.mergeEitherWay ExU.realMergeables
-  , testCases "real merge and commute" PropU.mergeCommute ExU.realMergeables
+  , testCases "read and show work on RepoPatchV2" PropU.show_read ExU.repov2Patches
+  , testCases "example flattenings work" PropU.consistentTreeFlattenings ExU.repov2PatchLoopExamples
+  , testCases "V2 merge input consistent" (PropU.mergeArgumentsConsistent isConsistent) ExU.repov2Mergeables
+  , testCases "V2 merge input is forward" (PropU.mergeArgumentsConsistent isForward) ExU.repov2Mergeables
+  , testCases "V2 merge output is forward" (PropU.mergeConsistent isForward) ExU.repov2Mergeables
+  , testCases "V2 merge output consistent" (PropU.mergeConsistent isConsistent) ExU.repov2Mergeables
+  , testCases "V2 merge either way" PropU.mergeEitherWay ExU.repov2Mergeables
+  , testCases "V2 merge and commute" PropU.mergeCommute ExU.repov2Mergeables
 
-  , testCases "real recommute" (PropU.recommute WSub.commute) ExU.realCommutables
-  , testCases "real inverses commute" (PropU.commuteInverses WSub.commute) ExU.realCommutables
-  , testCases "real permutivity" (PropU.permutivity WSub.commute) ExU.realNonduplicateTriples
-  , testCases "real partial permutivity" (PropU.partialPermutivity WSub.commute) ExU.realNonduplicateTriples
+  , testCases "V2 recommute" (PropU.recommute WSub.commute) ExU.repov2Commutables
+  , testCases "V2 inverses commute" (PropU.commuteInverses WSub.commute) ExU.repov2Commutables
+  , testCases "V2 permutivity" (PropU.permutivity WSub.commute) ExU.repov2NonduplicateTriples
+  , testCases "V2 partial permutivity" (PropU.partialPermutivity WSub.commute) ExU.repov2NonduplicateTriples
   ]
 
-instance PrimPatch prim => Check (RealPatch prim) where
+instance PrimPatch prim => Check (RepoPatchV2 prim) where
   checkPatch p = return $ isNothing $ isConsistent p
 
-instance Check V3.Prim where
+instance Check FileUUID.Prim where
   checkPatch _ = return True -- XXX
 
-commuteReals :: PrimPatch prim => (RealPatch prim :> RealPatch prim) wX wY -> Maybe ((RealPatch prim :> RealPatch prim) wX wY)
-commuteReals = commute
+commuteRepoPatchV2s :: PrimPatch prim => (RepoPatchV2 prim :> RepoPatchV2 prim) wX wY -> Maybe ((RepoPatchV2 prim :> RepoPatchV2 prim) wX wY)
+commuteRepoPatchV2s = commute
 
 qc_prim :: forall prim wX wY wA model.
            (PrimPatch prim, ArbitraryPrim prim, Show2 prim
@@ -183,8 +184,8 @@ qc_prim p =
   [ pair_properties            (undefined :: prim wX wY)    "arbitrary"    arbitraryThing'
   , pair_properties            (undefined :: FL prim wX wY) "arbitrary FL" arbitraryThing'
   , coalesce_properties        (undefined :: prim wX wY)    "arbitrary"    arbitraryThing'
-  , nonreal_commute_properties (undefined :: prim wX wY)    "arbitrary"    arbitraryThing'
-  , nonreal_commute_properties (undefined :: FL prim wX wY) "arbitrary FL" arbitraryThing'
+  , nonrpv2_commute_properties (undefined :: prim wX wY)    "arbitrary"    arbitraryThing'
+  , nonrpv2_commute_properties (undefined :: FL prim wX wY) "arbitrary FL" arbitraryThing'
   , patch_properties           (undefined :: prim wX wA)    "arbitrary"    arbitraryThing'
   , patch_properties           (undefined :: FL prim wX wA) "arbitrary FL" arbitraryThing'
   , patch_repo_properties      (undefined :: prim wX wA)    "arbitrary"    arbitraryThing'
@@ -197,41 +198,41 @@ qc_prim p =
 qc_V2P1 :: [Test]
 qc_V2P1 =
   [ testProperty "tree flattenings are consistent... "
-    PropR.propConsistentTreeFlattenings
-  , testProperty "with quickcheck that real patches are consistent... "
+    (PropR.propConsistentTreeFlattenings :: Sealed (WithStartState (ModelOf V1.Prim) (Tree V1.Prim)) -> Bool)
+  , testProperty "with quickcheck that RepoPatchV2 patches are consistent... "
     (unseal $ P1.patchFromTree $ fromMaybe . isConsistent)
   -- permutivity ----------------------------------------------------------------------------
   , testConditional "permutivity"
     (unseal $ P1.commuteTripleFromTree notDuplicatestriple)
-    (unseal $ P1.commuteTripleFromTree $ PropG.permutivity commuteReals)
+    (unseal $ P1.commuteTripleFromTree $ PropG.permutivity commuteRepoPatchV2s)
   , testConditional "partial permutivity"
     (unseal $ P1.commuteTripleFromTree notDuplicatestriple)
-    (unseal $ P1.commuteTripleFromTree $ PropG.partialPermutivity commuteReals)
+    (unseal $ P1.commuteTripleFromTree $ PropG.partialPermutivity commuteRepoPatchV2s)
   , testConditional "nontrivial permutivity"
     (unseal $ P1.commuteTripleFromTree (\t -> nontrivialTriple t && notDuplicatestriple t))
-    (unseal $ P1.commuteTripleFromTree $ (PropG.permutivity commuteReals))
+    (unseal $ P1.commuteTripleFromTree $ (PropG.permutivity commuteRepoPatchV2s))
   ]
 
 qc_V2 :: forall prim wXx wYy . (PrimPatch prim, Show1 (ModelOf prim), RepoModel (ModelOf prim),
-                                  Check (RealPatch prim), ArbitraryPrim prim, Show2 prim,
+                                  Check (RepoPatchV2 prim), ArbitraryPrim prim, Show2 prim,
                                   RepoState (ModelOf prim) ~ ApplyState prim)
       => prim wXx wYy -> [Test]
 qc_V2 _ =
-  [ testProperty "readPatch and showPatch work on RealPatch... "
-    (unseal $ patchFromTree $ (PropG.show_read :: RealPatch prim wX wY -> TestResult))
-  , testProperty "readPatch and showPatch work on FL RealPatch... "
-    (unseal2 $ (PropG.show_read :: FL (RealPatch prim) wX wY -> TestResult))
+  [ testProperty "readPatch and showPatch work on RepoPatchV2... "
+    (unseal $ patchFromTree $ (PropG.show_read :: RepoPatchV2 prim wX wY -> TestResult))
+  , testProperty "readPatch and showPatch work on FL RepoPatchV2... "
+    (unseal2 $ (PropG.show_read :: FL (RepoPatchV2 prim) wX wY -> TestResult))
   , testProperty "we can do merges using QuickCheck"
     (isNothing . (PropG.propIsMergeable ::
                      Sealed (WithStartState (ModelOf prim) (Tree prim))
-                     -> Maybe (Tree (RealPatch prim) wX)))
+                     -> Maybe (Tree (RepoPatchV2 prim) wX)))
   ]
   ++ concat
-  [ merge_properties   (undefined :: RealPatch prim wX wY) "tree" mergePairFromTree
-  , merge_properties   (undefined :: RealPatch prim wX wY) "twfp" mergePairFromTWFP
-  , pair_properties    (undefined :: RealPatch prim wX wY) "tree" commutePairFromTree
-  , pair_properties    (undefined :: RealPatch prim wX wY) "twfp" commutePairFromTWFP
-  , patch_properties   (undefined :: RealPatch prim wX wY) "tree" patchFromTree
+  [ merge_properties   (undefined :: RepoPatchV2 prim wX wY) "tree" mergePairFromTree
+  , merge_properties   (undefined :: RepoPatchV2 prim wX wY) "twfp" mergePairFromTWFP
+  , pair_properties    (undefined :: RepoPatchV2 prim wX wY) "tree" commutePairFromTree
+  , pair_properties    (undefined :: RepoPatchV2 prim wX wY) "twfp" commutePairFromTWFP
+  , patch_properties   (undefined :: RepoPatchV2 prim wX wY) "tree" patchFromTree
   ]
 
 properties :: forall thing gen. (Show1 gen, Arbitrary (Sealed gen)) =>
@@ -272,12 +273,12 @@ coalesce_properties p genname gen =
   properties gen "commute" genname
    (if runCoalesceTests p then [ ("coalesce commutes with commute", const True, PropG.coalesceCommute coalesce) ] else [])
 
--- The following properties do not hold for "Real" patches (conflictors and
+-- The following properties do not hold for "RepoPatchV2" patches (conflictors and
 -- duplicates, specifically) .
-nonreal_commute_properties :: forall p gen x y
+nonrpv2_commute_properties :: forall p gen x y
                             . (Show1 gen, Arbitrary (Sealed gen), Patchy p, ShowPatchBasic p, MyEq p)
                            => p x y -> PropList (p :> p) gen
-nonreal_commute_properties _ genname gen =
+nonrpv2_commute_properties _ genname gen =
   properties gen "commute" genname
   [ ("patch & inverse commute", const True       , PropG.patchAndInverseCommute commute)
   , ("patch & inverse commute", nontrivialCommute, PropG.patchAndInverseCommute commute)
@@ -356,13 +357,13 @@ qc_V1P1 =
   ] -- the following properties are disabled, because they routinely lead to
     -- exponential cases, making the tests run for ever and ever; nevertheless,
     -- we would expect them to hold
- {- ++ merge_properties (undefined :: V1.Patch Prim wX wY) "tree" mergePairFromTree
-    ++ merge_properties (undefined :: V1.Patch Prim wX wY) "twfp" mergePairFromTWFP
-    ++ commute_properties (undefined :: V1.Patch Prim wX wY) "tree" commutePairFromTree
-    ++ commute_properties (undefined :: V1.Patch Prim wX wY) "twfp" commutePairFromTWFP -}
+ {- ++ merge_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "tree" mergePairFromTree
+    ++ merge_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "twfp" mergePairFromTWFP
+    ++ commute_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "tree" commutePairFromTree
+    ++ commute_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "twfp" commutePairFromTWFP -}
 
 -- tests (either QuickCheck or Unit) that should be run on any type of patch
-general_patchTests :: (RepoPatch p, ArbitraryPrim (PrimOf p), Show2 (PrimOf p)) => PatchType p -> [Test]
+general_patchTests :: (RepoPatch p, ArbitraryPrim (PrimOf p), Show2 (PrimOf p)) => PatchType rt p -> [Test]
 general_patchTests pt =
      [ testGroup "Rebase patches" $ Rebase.testSuite pt
      ]
@@ -371,13 +372,14 @@ general_patchTests pt =
 testSuite :: [Test]
 testSuite = [ testGroup "Darcs.Patch.Prim.V1" $ qc_prim (undefined :: V1.Prim wX wY)
             , testGroup "Darcs.Patch.V1 (using Prim.V1)" $
-                unit_V1P1 ++ qc_V1P1 ++ general_patchTests (PatchType :: PatchType (V1.Patch V1.Prim))
+                unit_V1P1 ++ qc_V1P1 ++ general_patchTests (PatchType :: PatchType rt (V1.RepoPatchV1 V1.Prim))
             , testGroup "Darcs.Patch.V2 (using Prim.V1)" $
                 unit_V2P1 ++ qc_V2 (undefined :: V1.Prim wX wY) ++ qc_V2P1 ++
-                general_patchTests (PatchType :: PatchType (RealPatch V1.Prim))
-            -- , testGroup "Darcs.Patch.Prim.V3" $ qc_prim (undefined :: V3.Prim wX wY)
-            , testGroup "Darcs.Patch.V2 (using Prim.V3)" $
-                qc_V2 (undefined :: V3.Prim wX wY) ++
-                general_patchTests (PatchType :: PatchType (RealPatch V3.Prim))
+                general_patchTests (PatchType :: PatchType rt (RepoPatchV2 V1.Prim))
+            -- , testGroup "Darcs.Patch.Prim.FileUUID" $ qc_prim (undefined :: FileUUID.Prim wX wY)
+            , testGroup "Darcs.Patch.V2 (using Prim.FileUUID)" $
+                qc_V2 (undefined :: FileUUID.Prim wX wY) ++
+                general_patchTests (PatchType :: PatchType rt (RepoPatchV2 FileUUID.Prim))
             , Darcs.Test.Patch.Info.testSuite
+            , Darcs.Test.Patch.Selection.testSuite
             ]
