@@ -6,6 +6,10 @@ import qualified Darcs.Test.Patch.Arbitrary.Generic as T
      ( commuteTripleFromTree, commutePairFromTree, commutePairFromTWFP
      , mergePairFromTree, mergePairFromTWFP
      , patchFromTree )
+
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Test.Patch.Arbitrary.Generic
 import Darcs.Test.Patch.RepoModel
 
@@ -22,12 +26,12 @@ import Darcs.Patch.RepoPatch ( RepoPatch )
 import Darcs.Patch.FileHunk( IsHunk( isHunk ), FileHunk(..) )
 
 import Darcs.Test.Patch.V1Model
+import Darcs.Util.Path
 import Darcs.Test.Util.QuickCheck ( alpha, notIn, maybeOf )
 
 import Darcs.UI.Commands.Replace ( defaultToks )
 import Darcs.Patch.Prim
 
-import Control.Applicative ( (<$>) )
 import qualified Data.ByteString.Char8 as BC
 import Data.Maybe ( isJust )
 
