@@ -67,6 +67,9 @@ module Darcs.Patch.Match
     , MatchFlag(..)
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Text.ParserCombinators.Parsec
     ( parse
     , CharParser
@@ -100,25 +103,27 @@ import Data.Char ( toLower )
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.Patch
     ( Patchy
+    , IsRepoType
     , hunkMatches
     , listTouchedFiles
-    , patchcontents
-    , Named
     , invert
     , invertRL
-    , patch2patchinfo
     , apply
     )
 import Darcs.Patch.Info ( justName, justAuthor, justLog, makePatchname,
                           piDate )
+import Darcs.Patch.Named.Wrapped
+    ( WrappedNamed
+    , patch2patchinfo
+    )
 
 import qualified Data.ByteString.Char8 as BC
 
 import Darcs.Patch.Dummy ( DummyPatch )
 
 import Darcs.Patch.Matchable ( Matchable )
-import Darcs.Patch.MaybeInternal ( MaybeInternal, patchInternalChecker, isInternal, flIsInternal )
 import Darcs.Patch.MonadProgress ( MonadProgress )
+import Darcs.Patch.Named.Wrapped ( runInternalChecker, namedIsInternal, namedInternalChecker )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, conscientiously, hopefully )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, newset2RL, Origin )
 import Darcs.Patch.Type ( PatchType(..) )
@@ -127,7 +132,7 @@ import Darcs.Patch.ApplyPatches( applyPatches )
 import Darcs.Patch.Depends ( getPatchesBeyondTag, splitOnTag )
 
 import Darcs.Patch.Witnesses.Eq ( isIsEq )
-import Darcs.Patch.Witnesses.Ordered ( RL(..), consRLSealed, FL(..), (:>)(..) )
+import Darcs.Patch.Witnesses.Ordered ( RL(..), snocRLSealed, FL(..), (:>)(..) )
 import Darcs.Patch.Witnesses.Sealed
     ( FlippedSeal(..), Sealed2(..),
     seal, flipSeal, seal2, unsealFlipped, unseal2, unseal )
@@ -136,18 +141,18 @@ import Darcs.Patch.ApplyMonad ( ApplyMonad(..) )
 
 import Darcs.Util.DateMatcher ( parseDateMatcher )
 
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
 #include "impossible.h"
 
 -- | A type for predicates over patches which do not care about
 -- contexts
-type MatchFun p = Sealed2 (PatchInfoAnd p) -> Bool
+type MatchFun rt p = Sealed2 (PatchInfoAnd rt p) -> Bool
 
 -- | A @Matcher@ is made of a 'MatchFun' which we will use to match
 -- patches and a @String@ representing it.
-data Matcher p = MATCH String (MatchFun p)
+data Matcher rt p = MATCH String (MatchFun rt p)
 
-instance Show (Matcher p) where
+instance Show (Matcher rt p) where
     show (MATCH s _) = '"':s ++ "\""
 
 
@@ -172,37 +177,37 @@ data MatchFlag =
                  deriving ( Show )
 
 
-makeMatcher :: String -> (Sealed2 (PatchInfoAnd p) -> Bool) -> Matcher p
+makeMatcher :: String -> (Sealed2 (PatchInfoAnd rt p) -> Bool) -> Matcher rt p
 makeMatcher = MATCH
 
 -- | @applyMatcher@ applies a matcher to a patch.
-applyMatcher :: Matcher p -> PatchInfoAnd p wX wY -> Bool
+applyMatcher :: Matcher rt p -> PatchInfoAnd rt p wX wY -> Bool
 applyMatcher (MATCH _ m) = m . seal2
 
-parseMatch :: Matchable p => String -> Either String (MatchFun p)
+parseMatch :: Matchable p => String -> Either String (MatchFun rt p)
 parseMatch pattern =
     case parse matchParser "match" pattern of
     Left err -> Left $ "Invalid --match pattern '"++ pattern ++
                 "'.\n"++ unlines (map ("    "++) $ lines $ show err) -- indent
     Right m -> Right m
 
-matchPattern :: Matchable p => String -> Matcher p
+matchPattern :: Matchable p => String -> Matcher rt p
 matchPattern pattern =
     case parseMatch pattern of
     Left err -> error err
     Right m -> makeMatcher pattern m
 
-addInternalMatcher :: Matchable p => Maybe (Matcher p) -> Maybe (Matcher p)
+addInternalMatcher :: (IsRepoType rt, Matchable p) => Maybe (Matcher rt p) -> Maybe (Matcher rt p)
 addInternalMatcher om =
-  case patchInternalChecker of
+  case namedInternalChecker of
     Nothing -> om
     Just f ->
-         let matchFun = unseal2 (not . isIsEq . isInternal f . patchcontents . hopefully)
+         let matchFun = unseal2 (not . isIsEq . runInternalChecker f . hopefully)
          in case om of
             Nothing -> Just (MATCH "internal patch" matchFun)
             Just (MATCH s oldFun) -> Just (MATCH s (\p -> matchFun p && oldFun p))
 
-matchParser :: Matchable p => CharParser st (MatchFun p)
+matchParser :: Matchable p => CharParser st (MatchFun rt p)
 matchParser = submatcher <?> helpfulErrorMsg
   where
     submatcher = do
@@ -217,18 +222,18 @@ matchParser = submatcher <?> helpfulErrorMsg
                       ++ "\nfor more help, see `darcs help patterns`."
 
     -- This type signature is just to bind an ambiguous type var.
-    ps :: [(String, String, [String], String -> MatchFun DummyPatch)]
+    ps :: [(String, String, [String], String -> MatchFun rt DummyPatch)]
     ps = primitiveMatchers
 
     -- matchAnyPatch is returned if submatch fails without consuming any
     -- input, i.e. if we pass --match '', we want to match anything.
-    matchAnyPatch :: Matchable p => MatchFun p
+    matchAnyPatch :: Matchable p => MatchFun rt p
     matchAnyPatch = const True
 
-submatch :: Matchable p => CharParser st (MatchFun p)
+submatch :: Matchable p => CharParser st (MatchFun rt p)
 submatch = buildExpressionParser table match
 
-table :: OperatorTable Char st (MatchFun p)
+table :: OperatorTable Char st (MatchFun rt p)
 table   = [ [prefix "not" negate_match,
              prefix "!" negate_match ]
           , [binary "||" or_match,
@@ -248,13 +253,13 @@ table   = [ [prefix "not" negate_match,
 trystring :: String -> CharParser st String
 trystring s = try $ string s
 
-match :: Matchable p => CharParser st (MatchFun p)
+match :: Matchable p => CharParser st (MatchFun rt p)
 match = between spaces spaces (parens submatch <|> choice matchers_)
   where
     matchers_ = map createMatchHelper primitiveMatchers
 
-createMatchHelper :: (String, String, [String], String -> MatchFun p)
-                  -> CharParser st (MatchFun p)
+createMatchHelper :: (String, String, [String], String -> MatchFun rt p)
+                  -> CharParser st (MatchFun rt p)
 createMatchHelper (key,_,_,matcher) =
   do _ <- trystring key
      spaces
@@ -278,7 +283,7 @@ helpOnMatchers =
    "- --hash=HASH is a synonym for --matches='hash HASH'",
    "- --from-patch and --to-patch are synonyms for --from-match='name... and --to-match='name...",
    "- --from-patch and --to-match can be unproblematically combined:",
-   "  `darcs changes --from-patch='html.*documentation' --to-match='date 20040212'`",
+   "  `darcs log --from-patch='html.*documentation' --to-match='date 20040212'`",
    "",
    "The following primitive Boolean expressions are supported:"
    ,""]
@@ -286,7 +291,7 @@ helpOnMatchers =
   ++ ["", "Here are some examples:", ""]
   ++ examples
   where -- This type signature exists to appease GHC.
-        ps :: [(String, String, [String], String -> MatchFun DummyPatch)]
+        ps :: [(String, String, [String], String -> MatchFun rt DummyPatch)]
         ps = primitiveMatchers
         keywords = [showKeyword k d | (k,d,_,_) <- ps]
         examples = [showExample k e | (k,_,es,_) <- ps, e <- es]
@@ -296,13 +301,10 @@ helpOnMatchers =
             -- or "exact STRING - match against exact patch name".
             "- " ++ keyword ++ " - " ++ description ++ "."
         showExample keyword example =
-            -- FIXME: this string is long, and its not a use case I've
-            -- ever seen in practice.  Can we use something else,
-            -- like "darcs changes --matches"? --twb, 2008-12-28
-            "    darcs annotate --summary --match "
+            "    darcs log --match "
             ++ "'" ++ keyword ++ " " ++ example ++ "'"
 
-primitiveMatchers :: Matchable p => [(String, String, [String], String -> MatchFun p)]
+primitiveMatchers :: Matchable p => [(String, String, [String], String -> MatchFun rt p)]
                      -- ^ keyword (operator), help description, list
                      -- of examples, matcher function
 primitiveMatchers =
@@ -331,8 +333,8 @@ primitiveMatchers =
           , ["src/foo.c", "src/", "\"src/*.(c|h)\""]
           , touchmatch ) ]
 
-parens :: CharParser st (MatchFun p)
-       -> CharParser st (MatchFun p)
+parens :: CharParser st (MatchFun rt p)
+       -> CharParser st (MatchFun rt p)
 parens = between (string "(") (string ")")
 
 quoted :: CharParser st String
@@ -345,7 +347,7 @@ quoted = between (char '"') (char '"')
          <?> "string"
 
 namematch, exactmatch, authormatch, hunkmatch, hashmatch, datematch, touchmatch
-  :: Matchable p => String -> MatchFun p
+  :: Matchable p => String -> MatchFun rt p
 
 namematch r (Sealed2 hp) = isJust $ matchRegex (mkRegex r) $ justName (info hp)
 
@@ -353,12 +355,11 @@ exactmatch r (Sealed2 hp) = r == justName (info hp)
 
 authormatch a (Sealed2 hp) = isJust $ matchRegex (mkRegex a) $ justAuthor (info hp)
 
-logmatch :: Matchable p => String -> MatchFun p
+logmatch :: Matchable p => String -> MatchFun rt p
 logmatch l (Sealed2 hp) = isJust $ matchRegex (mkRegex l) $ justLog (info hp)
 
-hunkmatch r (Sealed2 hp) = let patch = patchcontents $ hopefully hp
-                               regexMatcher = isJust . matchRegex (mkRegex r) . BC.unpack
-                           in hunkMatches regexMatcher patch
+hunkmatch r (Sealed2 hp) = let regexMatcher = isJust . matchRegex (mkRegex r) . BC.unpack
+                           in hunkMatches regexMatcher hp
 
 hashmatch h (Sealed2 hp) = let rh = show $ makePatchname (info hp)
                                lh = map toLower h
@@ -367,7 +368,7 @@ hashmatch h (Sealed2 hp) = let rh = show $ makePatchname (info hp)
 datematch d (Sealed2 hp) = let dm = unsafePerformIO $ parseDateMatcher d
                                   in dm $ piDate (info hp)
 
-touchmatch r (Sealed2 hp) = let files = listTouchedFiles $ patchcontents $ hopefully hp
+touchmatch r (Sealed2 hp) = let files = listTouchedFiles hp
                             in any (isJust . matchRegex (mkRegex r)) files
 
 data InclusiveOrExclusive = Inclusive | Exclusive deriving Eq
@@ -376,25 +377,30 @@ data InclusiveOrExclusive = Inclusive | Exclusive deriving Eq
 -- @flags@ which corresponds to a match that is "non-range". Thus,
 -- @--match@, @--patch@, @--hash@ and @--index@ make @haveNonrangeMatch@
 -- true, but not @--from-patch@ or @--to-patch@.
-haveNonrangeMatch :: forall p . Matchable p => PatchType p -> [MatchFlag] -> Bool
+haveNonrangeMatch :: forall rt p . (IsRepoType rt, Matchable p) => PatchType rt p -> [MatchFlag] -> Bool
 haveNonrangeMatch _ fs =
      case hasIndexRange fs of Just (m,n) | m == n -> True; _ -> False
-  || isJust (nonrangeMatcher fs::Maybe (Matcher p))
+  || isJust (nonrangeMatcher fs::Maybe (Matcher rt p))
 
 -- | @havePatchsetMatch flags@ tells whether there is a "patchset
 -- match" in the flag list. A patchset match is @--match@ or
 -- @--patch@, or @--context@, but not @--from-patch@ nor (!)
 -- @--index@.
 -- Question: Is it supposed not to be a subset of @haveNonrangeMatch@?
-havePatchsetMatch :: [MatchFlag] -> Bool
-havePatchsetMatch fs = isJust (nonrangeMatcher fs::Maybe (Matcher DummyPatch)) || hasC fs
+havePatchsetMatch
+  :: forall rt p
+   . (IsRepoType rt, Matchable p)
+  => PatchType rt p -> [MatchFlag] -> Bool
+havePatchsetMatch _ fs = isJust (nonrangeMatcher fs::Maybe (Matcher rt p)) || hasC fs
     where hasC [] = False
           hasC (Context _:_) = True
           hasC (_:xs) = hasC xs
 
-getNonrangeMatchS :: (ApplyMonad m (ApplyState p), MonadProgress m, Matchable p, ApplyState p ~ Tree)
+getNonrangeMatchS :: ( ApplyMonad (ApplyState p) m, MonadProgress m
+                     , IsRepoType rt, Matchable p, ApplyState p ~ Tree
+                     )
                   => [MatchFlag]
-                  -> PatchSet p Origin wX
+                  -> PatchSet rt p Origin wX
                   -> m ()
 getNonrangeMatchS fs repo =
     case nonrangeMatcher fs of
@@ -408,11 +414,11 @@ getNonrangeMatchS fs repo =
 -- than against all patches since the creation of the repository.
 firstMatch :: [MatchFlag] -> Bool
 firstMatch fs = isJust (hasLastn fs)
-                 || isJust (firstMatcher fs::Maybe (Matcher DummyPatch))
+                 || isJust (firstMatcher fs::Maybe (Matcher rt DummyPatch))
                  || isJust (hasIndexRange fs)
 
-getFirstMatchS :: (ApplyMonad m (ApplyState p), MonadProgress m, Matchable p)
-               => [MatchFlag] -> PatchSet p Origin wX -> m ()
+getFirstMatchS :: (ApplyMonad (ApplyState p) m, MonadProgress m, Matchable p, IsRepoType rt)
+               => [MatchFlag] -> PatchSet rt p Origin wX -> m ()
 getFirstMatchS fs repo =
     case hasLastn fs of
     Just n -> unpullLastN repo n
@@ -430,10 +436,10 @@ getFirstMatchS fs repo =
 -- is if we match against patches up to a point in the past on, rather
 -- than against all patches until now.
 secondMatch :: [MatchFlag] -> Bool
-secondMatch fs = isJust (secondMatcher fs::Maybe (Matcher DummyPatch)) || isJust (hasIndexRange fs)
+secondMatch fs = isJust (secondMatcher fs::Maybe (Matcher rt DummyPatch)) || isJust (hasIndexRange fs)
 
-unpullLastN :: (ApplyMonad m (ApplyState p), MonadProgress m, Patchy p, MaybeInternal p)
-            => PatchSet p wX wY
+unpullLastN :: (ApplyMonad (ApplyState p) m, MonadProgress m, Patchy p, IsRepoType rt)
+            => PatchSet rt p wX wY
             -> Int
             -> m ()
 unpullLastN repo n = applyInvRL `unsealFlipped` safetake n (newset2RL repo)
@@ -442,7 +448,7 @@ checkMatchSyntax :: [MatchFlag] -> IO ()
 checkMatchSyntax opts =
  case getMatchPattern opts of
   Nothing -> return ()
-  Just p  -> either fail (const $ return ()) (parseMatch p::Either String (MatchFun DummyPatch))
+  Just p  -> either fail (const $ return ()) (parseMatch p::Either String (MatchFun rt DummyPatch))
 
 getMatchPattern :: [MatchFlag] -> Maybe String
 getMatchPattern [] = Nothing
@@ -450,16 +456,16 @@ getMatchPattern (OnePattern m:_) = Just m
 getMatchPattern (SeveralPattern m:_) = Just m
 getMatchPattern (_:fs) = getMatchPattern fs
 
-tagmatch :: Matchable p => String -> Matcher p
+tagmatch :: Matchable p => String -> Matcher rt p
 tagmatch r = makeMatcher ("tag-name "++r) tm
     where tm (Sealed2 p) =
               let n = justName (info p) in
               "TAG " `isPrefixOf` n && isJust (matchRegex (mkRegex r) $ drop 4 n)
 
-patchmatch :: Matchable p => String -> Matcher p
+patchmatch :: Matchable p => String -> Matcher rt p
 patchmatch r = makeMatcher ("patch-name "++r) (namematch r)
 
-hashmatch' :: Matchable p => String -> Matcher p
+hashmatch' :: Matchable p => String -> Matcher rt p
 hashmatch' r = makeMatcher ("hash "++r) (hashmatch r)
 
 
@@ -472,7 +478,8 @@ strictJust x = Just $! x
 -- | @nonrangeMatcher@ is the criterion that is used to match against
 -- patches in the interval. It is 'Just m' when the @--patch@, @--match@,
 -- @--tag@ options are passed (or their plural variants).
-nonrangeMatcher, nonrangeMatcherArgs :: Matchable p => [MatchFlag] -> Maybe (Matcher p)
+nonrangeMatcher :: (IsRepoType rt, Matchable p) => [MatchFlag] -> Maybe (Matcher rt p)
+nonrangeMatcherArgs :: Matchable p => [MatchFlag] -> Maybe (Matcher rt p)
 
 nonrangeMatcher fs = addInternalMatcher $ nonrangeMatcherArgs fs
 
@@ -496,7 +503,7 @@ nonrangeMatcherIsTag (_:fs) = nonrangeMatcherIsTag fs
 -- This left bound is also specified when we use the singular versions
 -- of @--patch@, @--match@ and @--tag@. Otherwise, @firstMatcher@
 -- returns @Nothing@.
-firstMatcher :: Matchable p => [MatchFlag] -> Maybe (Matcher p)
+firstMatcher :: Matchable p => [MatchFlag] -> Maybe (Matcher rt p)
 firstMatcher [] = Nothing
 firstMatcher (OnePattern m:_) = strictJust $ matchPattern m
 firstMatcher (AfterPattern m:_) = strictJust $ matchPattern m
@@ -512,7 +519,7 @@ firstMatcherIsTag [] = False
 firstMatcherIsTag (AfterTag _:_) = True
 firstMatcherIsTag (_:fs) = firstMatcherIsTag fs
 
-secondMatcher :: Matchable p => [MatchFlag] -> Maybe (Matcher p)
+secondMatcher :: Matchable p => [MatchFlag] -> Maybe (Matcher rt p)
 secondMatcher [] = Nothing
 secondMatcher (OnePattern m:_) = strictJust $ matchPattern m
 secondMatcher (UpToPattern m:_) = strictJust $ matchPattern m
@@ -530,20 +537,20 @@ secondMatcherIsTag (_:fs) = secondMatcherIsTag fs
 
 -- | @matchAPatchread fs p@ tells whether @p@ matches the matchers in
 -- the flags listed in @fs@.
-matchAPatchread :: Matchable p => [MatchFlag] -> PatchInfoAnd p wX wY -> Bool
+matchAPatchread :: (IsRepoType rt, Matchable p) => [MatchFlag] -> PatchInfoAnd rt p wX wY -> Bool
 matchAPatchread fs = case nonrangeMatcher fs of
                        Nothing -> const True
                        Just m -> applyMatcher m
 
 -- | @matchAPatch fs p@ tells whether @p@ matches the matchers in
 -- the flags @fs@
-matchAPatch :: Matchable p => [MatchFlag] -> PatchInfoAnd p wX wY -> Bool
+matchAPatch :: (IsRepoType rt, Matchable p) => [MatchFlag] -> PatchInfoAnd rt p wX wY -> Bool
 matchAPatch fs p =
     case nonrangeMatcher fs of
     Nothing -> True
     Just m -> applyMatcher m p
 
-matchPatch :: Matchable p => [MatchFlag] -> PatchSet p wStart wX -> Sealed2 (Named p)
+matchPatch :: (IsRepoType rt, Matchable p) => [MatchFlag] -> PatchSet rt p wStart wX -> Sealed2 (WrappedNamed rt p)
 matchPatch fs ps =
     case hasIndexRange fs of
     Just (a,a') | a == a' -> case unseal myhead $ dropn (a-1) ps of
@@ -551,9 +558,9 @@ matchPatch fs ps =
                              Nothing -> error "Patch out of range!"
                 | otherwise -> bug ("Invalid index range match given to matchPatch: "++
                                     show (PatchIndexRange a a'))
-                where myhead :: PatchSet p wStart wX -> Maybe (Sealed2 (PatchInfoAnd p))
-                      myhead (PatchSet NilRL (Tagged t _ _ :<: _)) = Just $ seal2 t
-                      myhead (PatchSet (x:<:_) _) = Just $ seal2 x
+                where myhead :: PatchSet rt p wStart wX -> Maybe (Sealed2 (PatchInfoAnd rt p))
+                      myhead (PatchSet (_ :<: Tagged t _ _) NilRL) = Just $ seal2 t
+                      myhead (PatchSet _ (_:<:x)) = Just $ seal2 x
                       myhead _ = Nothing
     Nothing -> case nonrangeMatcher fs of
                     Nothing -> bug "Couldn't matchPatch"
@@ -575,8 +582,8 @@ hasIndexRange (_:fs) = hasIndexRange fs
 -- first matcher, ie the one that comes first dependencywise. Hence,
 -- patches in @matchFirstPatchset fs ps@ are the context for the ones
 -- we don't want.
-matchFirstPatchset :: Matchable p => [MatchFlag] -> PatchSet p wStart wX
-                   -> SealedPatchSet p wStart
+matchFirstPatchset :: (IsRepoType rt, Matchable p) => [MatchFlag] -> PatchSet rt p wStart wX
+                   -> SealedPatchSet rt p wStart
 matchFirstPatchset fs patchset =
     case hasLastn fs of
     Just n -> dropn n patchset
@@ -591,19 +598,19 @@ matchFirstPatchset fs patchset =
                                             else matchAPatchset m patchset
 
 -- | @dropn n ps@ drops the @n@ last patches from @ps@.
-dropn :: MaybeInternal p => Int -> PatchSet p wStart wX -> SealedPatchSet p wStart
+dropn :: IsRepoType rt => Int -> PatchSet rt p wStart wX -> SealedPatchSet rt p wStart
 dropn n ps | n <= 0 = seal ps
-dropn n (PatchSet NilRL (Tagged t _ ps :<: ts)) = dropn n $ PatchSet (t:<:ps) ts
+dropn n (PatchSet (ts :<: Tagged t _ ps) NilRL) = dropn n $ PatchSet ts (ps:<:t)
 dropn _ (PatchSet NilRL NilRL) = seal $ PatchSet NilRL NilRL
-dropn n (PatchSet (p:<:ps) ts)
-    | isIsEq (flIsInternal (patchcontents (hopefully p)))
-   = dropn n $ PatchSet ps ts
-dropn n (PatchSet (_:<:ps) ts) = dropn (n-1) $ PatchSet ps ts
+dropn n (PatchSet ts (ps:<:p))
+    | isIsEq (namedIsInternal (hopefully p))
+   = dropn n $ PatchSet ts ps
+dropn n (PatchSet ts (ps:<:_)) = dropn (n-1) $ PatchSet ts ps
 
 -- | @matchSecondPatchset fs ps@ returns the part of @ps@ before its
 -- second matcher, ie the one that comes last dependencywise.
-matchSecondPatchset :: Matchable p => [MatchFlag] -> PatchSet p wStart wX
-                    -> SealedPatchSet p wStart
+matchSecondPatchset :: (IsRepoType rt, Matchable p) => [MatchFlag] -> PatchSet rt p wStart wX
+                    -> SealedPatchSet rt p wStart
 matchSecondPatchset fs ps =
   case hasIndexRange fs of
   Just (a,_) -> dropn (a-1) ps
@@ -618,7 +625,7 @@ matchSecondPatchset fs ps =
 -- the earliest patch in a sequence, as opposed to 'matchSecondPatchset' which picks up the
 -- first match starting from the latest patch
 splitSecondFL :: Matchable p
-              => (forall wA wB . q wA wB -> Sealed2 (PatchInfoAnd p))
+              => (forall wA wB . q wA wB -> Sealed2 (PatchInfoAnd rt p))
               -> [MatchFlag]
               -> FL q wX wY
               -> (FL q :> FL q) wX wY -- ^The first element is the patches before and including the first patch matching the second matcher,
@@ -634,38 +641,38 @@ splitSecondFL extract fs ps =
 
 -- | @findAPatch m ps@ returns the last patch in @ps@ matching @m@, and
 -- calls 'error' if there is none.
-findAPatch :: Matchable p => Matcher p -> PatchSet p wStart wX -> Sealed2 (Named p)
+findAPatch :: Matchable p => Matcher rt p -> PatchSet rt p wStart wX -> Sealed2 (WrappedNamed rt p)
 findAPatch m (PatchSet NilRL NilRL) = error $ "Couldn't find patch matching " ++ show m
-findAPatch m (PatchSet NilRL (Tagged t _ ps :<: ts)) = findAPatch m (PatchSet (t:<:ps) ts)
-findAPatch m (PatchSet (p:<:ps) ts) | applyMatcher m p = seal2 $ hopefully p
-                                    | otherwise = findAPatch m (PatchSet ps ts)
+findAPatch m (PatchSet (ts :<: Tagged t _ ps) NilRL) = findAPatch m (PatchSet ts (ps:<:t))
+findAPatch m (PatchSet ts (ps:<:p)) | applyMatcher m p = seal2 $ hopefully p
+                                    | otherwise = findAPatch m (PatchSet ts ps)
 
 -- | @matchAPatchset m ps@ returns a (the largest?) subset of @ps@
 -- ending in patch which matches @m@. Calls 'error' if there is none.
-matchAPatchset :: Matchable p => Matcher p -> PatchSet p wStart wX
-               -> SealedPatchSet p wStart
+matchAPatchset :: Matchable p => Matcher rt p -> PatchSet rt p wStart wX
+               -> SealedPatchSet rt p wStart
 matchAPatchset m (PatchSet NilRL NilRL) = error $ "Couldn't find patch matching " ++ show m
-matchAPatchset m (PatchSet NilRL (Tagged t _ ps :<: ts)) = matchAPatchset m (PatchSet (t:<:ps) ts)
-matchAPatchset m (PatchSet (p:<:ps) ts) | applyMatcher m p = seal (PatchSet (p:<:ps) ts)
-                                        | otherwise = matchAPatchset m (PatchSet ps ts)
+matchAPatchset m (PatchSet (ts :<: Tagged t _ ps) NilRL) = matchAPatchset m (PatchSet ts (ps:<:t))
+matchAPatchset m (PatchSet ts (ps:<:p)) | applyMatcher m p = seal (PatchSet ts (ps:<:p))
+                                        | otherwise = matchAPatchset m (PatchSet ts ps)
 
 -- | @getMatchingTag m ps@, where @m@ is a 'Matcher' which matches tags
 -- returns a 'SealedPatchSet' containing all patches in the last tag which
 -- matches @m@. Last tag means the most recent tag in repository order,
--- i.e. the last one you'd see if you ran darcs changes -t @m@. Calls
+-- i.e. the last one you'd see if you ran darcs log -t @m@. Calls
 -- 'error' if there is no matching tag.
-getMatchingTag :: Matchable p => Matcher p -> PatchSet p wStart wX -> SealedPatchSet p wStart
+getMatchingTag :: Matchable p => Matcher rt p -> PatchSet rt p wStart wX -> SealedPatchSet rt p wStart
 getMatchingTag m (PatchSet NilRL NilRL) = error $ "Couldn't find a tag matching " ++ show m
-getMatchingTag m (PatchSet NilRL (Tagged t _ ps :<: ts)) = getMatchingTag m (PatchSet (t:<:ps) ts)
-getMatchingTag m (PatchSet (p:<:ps) ts)
+getMatchingTag m (PatchSet (ts :<: Tagged t _ ps) NilRL) = getMatchingTag m (PatchSet ts (ps:<:t))
+getMatchingTag m (PatchSet ts (ps:<:p))
     | applyMatcher m p =
         -- found a non-clean tag, need to commute out the things that it doesn't depend on
-        case splitOnTag (info p) (PatchSet (p:<:ps) ts) of
+        case splitOnTag (info p) (PatchSet ts (ps:<:p)) of
             Nothing -> bug "splitOnTag couldn't find tag we explicitly provided!"
             Just (patchSet :> _) -> seal patchSet
-    | otherwise = getMatchingTag m (PatchSet ps ts)
+    | otherwise = getMatchingTag m (PatchSet ts ps)
 
-splitMatchFL :: Matchable p => (forall wA wB . q wA wB -> Sealed2 (PatchInfoAnd p)) -> Matcher p -> FL q wX wY -> (FL q :> FL q) wX wY
+splitMatchFL :: Matchable p => (forall wA wB . q wA wB -> Sealed2 (PatchInfoAnd rt p)) -> Matcher rt p -> FL q wX wY -> (FL q :> FL q) wX wY
 splitMatchFL _extract m NilFL = error $ "Couldn't find patch matching " ++ show m
 splitMatchFL extract m (p :>: ps)
    | unseal2 (applyMatcher m) . extract $ p = (p :>: NilFL) :> ps
@@ -674,40 +681,40 @@ splitMatchFL extract m (p :>: ps)
 
 -- | @matchExists m ps@ tells whether there is a patch matching
 -- @m@ in @ps@
-matchExists :: Matcher p -> PatchSet p wStart wX -> Bool
+matchExists :: Matcher rt p -> PatchSet rt p wStart wX -> Bool
 matchExists _ (PatchSet NilRL NilRL) = False
-matchExists m (PatchSet NilRL (Tagged t _ ps :<: ts)) = matchExists m (PatchSet (t:<:ps) ts)
-matchExists m (PatchSet (p:<:ps) ts) | applyMatcher m p = True
-                                     | otherwise = matchExists m (PatchSet ps ts)
+matchExists m (PatchSet (ts :<: Tagged t _ ps) NilRL) = matchExists m (PatchSet ts (ps:<:t))
+matchExists m (PatchSet ts (ps:<:p)) | applyMatcher m p = True
+                                     | otherwise = matchExists m (PatchSet ts ps)
 
-applyInvToMatcher :: (Matchable p, ApplyMonad m (ApplyState p))
-                  => InclusiveOrExclusive -> Matcher p -> PatchSet p Origin wX -> m ()
+applyInvToMatcher :: (Matchable p, ApplyMonad (ApplyState p) m)
+                  => InclusiveOrExclusive -> Matcher rt p -> PatchSet rt p Origin wX -> m ()
 applyInvToMatcher _ _ (PatchSet NilRL NilRL) = impossible
-applyInvToMatcher ioe m (PatchSet NilRL (Tagged t _ ps :<: ts)) = applyInvToMatcher ioe m
-                                                                  (PatchSet (t:<:ps) ts)
-applyInvToMatcher ioe m (PatchSet (p:<:ps) xs)
+applyInvToMatcher ioe m (PatchSet (ts :<: Tagged t _ ps) NilRL) = applyInvToMatcher ioe m
+                                                                  (PatchSet ts (ps:<:t))
+applyInvToMatcher ioe m (PatchSet xs (ps:<:p))
     | applyMatcher m p = when (ioe == Inclusive) (applyInvp p)
-    | otherwise = applyInvp p >> applyInvToMatcher ioe m (PatchSet ps xs)
+    | otherwise = applyInvp p >> applyInvToMatcher ioe m (PatchSet xs ps)
 
 -- | @applyNInv@ n ps applies the inverse of the last @n@ patches of @ps@.
-applyNInv :: (Matchable p, ApplyMonad m (ApplyState p)) => Int -> PatchSet p Origin wX -> m ()
+applyNInv :: (Matchable p, ApplyMonad (ApplyState p) m) => Int -> PatchSet rt p Origin wX -> m ()
 applyNInv n _ | n <= 0 = return ()
 applyNInv _ (PatchSet NilRL NilRL) = error "Index out of range."
-applyNInv n (PatchSet NilRL (Tagged t _ ps :<: ts)) =
-  applyNInv n (PatchSet (t :<: ps) ts)
-applyNInv n (PatchSet (p :<: ps) xs) =
-  applyInvp p >> applyNInv (n - 1) (PatchSet ps xs)
+applyNInv n (PatchSet (ts :<: Tagged t _ ps) NilRL) =
+  applyNInv n (PatchSet ts (ps :<: t))
+applyNInv n (PatchSet xs (ps :<: p)) =
+  applyInvp p >> applyNInv (n - 1) (PatchSet xs ps)
 
 
-getMatcherS :: (ApplyMonad m (ApplyState p), Matchable p) =>
-                 InclusiveOrExclusive -> Matcher p -> PatchSet p Origin wX -> m ()
+getMatcherS :: (ApplyMonad (ApplyState p) m, Matchable p) =>
+                 InclusiveOrExclusive -> Matcher rt p -> PatchSet rt p Origin wX -> m ()
 getMatcherS ioe m repo =
     if matchExists m repo
     then applyInvToMatcher ioe m repo
     else fail $ "Couldn't match pattern "++ show m
 
-getTagS :: (ApplyMonad m (ApplyState p), MonadProgress m, Matchable p) =>
-             Matcher p -> PatchSet p Origin wX -> m ()
+getTagS :: (ApplyMonad (ApplyState p) m, MonadProgress m, Matchable p) =>
+             Matcher rt p -> PatchSet rt p Origin wX -> m ()
 getTagS matcher repo = do
     let pinfo = patch2patchinfo `unseal2` findAPatch matcher repo
     case getPatchesBeyondTag pinfo repo of
@@ -715,23 +722,22 @@ getTagS matcher repo = do
 
 -- | @applyInvp@ tries to get the patch that's in a 'PatchInfoAnd
 -- patch', and to apply its inverse. If we fail to fetch the patch
--- (presumably in a partial repositiory), then we share our sorrow
--- with the user.
-applyInvp :: (Patchy p, ApplyMonad m (ApplyState p)) => PatchInfoAnd p wX wY -> m ()
+-- then we share our sorrow with the user.
+applyInvp :: (Patchy p, ApplyMonad (ApplyState p) m) => PatchInfoAnd rt p wX wY -> m ()
 applyInvp hp = apply (invert $ fromHopefully hp)
     where fromHopefully = conscientiously $ \e ->
-                     text "Sorry, partial repository problem.  Patch not available:"
+                     text "Sorry, patch not available:"
                      $$ e
                      $$ text ""
                      $$ text "If you think what you're trying to do is ok then"
                      $$ text "report this as a bug on the darcs-user list."
 
 -- | a version of 'take' for 'RL' lists that cater for contexts.
-safetake :: MaybeInternal p => Int -> RL (PatchInfoAnd p) wX wY -> FlippedSeal (RL (PatchInfoAnd p)) wY
+safetake :: IsRepoType rt => Int -> RL (PatchInfoAnd rt p) wX wY -> FlippedSeal (RL (PatchInfoAnd rt p)) wY
 safetake 0 _ = flipSeal NilRL
 safetake _ NilRL = error "There aren't that many patches..."
-safetake i (a:<:as) | isIsEq (flIsInternal (patchcontents (hopefully a))) = a `consRLSealed` safetake i as
-safetake i (a:<:as) = a `consRLSealed` safetake (i-1) as
+safetake i (as:<:a) | isIsEq (namedIsInternal (hopefully a)) = safetake i as `snocRLSealed` a
+safetake i (as:<:a) = safetake (i-1) as `snocRLSealed` a
 
-applyInvRL :: (ApplyMonad m (ApplyState p), MonadProgress m, Patchy p) => RL (PatchInfoAnd p) wX wR -> m ()
+applyInvRL :: (ApplyMonad (ApplyState p) m, MonadProgress m, Patchy p) => RL (PatchInfoAnd rt p) wX wR -> m ()
 applyInvRL = applyPatches . invertRL -- this gives nicer feedback
