@@ -4,7 +4,7 @@ module Darcs.Patch.Prim
          primIsAddfile, primIsHunk, primIsBinary, primIsSetpref,
          primIsAdddir, is_filepatch,
          canonize, tryToShrink,
-         sortCoalesceFL, join, canonizeFL,
+         sortCoalesceFL, coalesce, canonizeFL,
          tryShrinkingInverse,
          summarizePrim,
          applyPrimFL,
