@@ -8,6 +8,9 @@ module Darcs.Patch.Prim.Class
     )
     where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.ApplyMonad ( ApplyMonad )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.FileHunk ( FileHunk, IsHunk )
@@ -23,6 +26,7 @@ import Darcs.Patch.SummaryData ( SummDetail )
 import Darcs.Patch.Witnesses.Eq ( MyEq(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), RL, (:>), mapFL, mapFL_FL, reverseFL )
+import Darcs.Patch.Witnesses.Show ( Show2 )
 import Darcs.Patch.Witnesses.Sealed ( Sealed )
 
 import Darcs.Util.Printer ( Doc, vcat )
@@ -34,6 +38,7 @@ import qualified Data.ByteString as B ( ByteString )
 class (Patchy prim, MyEq prim
       ,PatchListFormat prim, IsHunk prim, RepairToFL prim
       ,PatchInspect prim, ReadPatch prim, ShowPatch prim
+      ,Show2 prim
       ,PrimConstruct prim, PrimCanonize prim
       ,PrimClassify prim, PrimDetails prim
       ,PrimShow prim, PrimRead prim, PrimApply prim
@@ -146,4 +151,4 @@ class PrimRead prim where
    readPrim :: ParserM m => FileNameFormat -> m (Sealed (prim wX))
 
 class PrimApply prim where
-   applyPrimFL :: ApplyMonad m (ApplyState prim) => FL prim wX wY -> m ()
+   applyPrimFL :: ApplyMonad (ApplyState prim) m => FL prim wX wY -> m ()
