@@ -22,7 +22,10 @@ module Darcs.Patch.TouchesFiles ( lookTouch, chooseTouching, choosePreTouching,
                       selectTouching,
                       deselectNotTouching, selectNotTouching,
                     ) where
-import Control.Applicative ( (<$>) )
+
+import Prelude ()
+import Darcs.Prelude
+
 import Data.List ( isSuffixOf, nub )
 
 import Darcs.Patch.Choices ( PatchChoices, Label, LabelledPatch,
@@ -34,7 +37,7 @@ import Darcs.Patch.Apply ( ApplyState, applyToFilePaths, effectOnFilePaths )
 import Darcs.Patch.Inspect ( PatchInspect )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), mapFL_FL, (+>+) )
 import Darcs.Patch.Witnesses.Sealed ( Sealed, seal )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 
 labelTouching :: (Patchy p, PatchInspect p, ApplyState p ~ Tree) => Bool
             -> [FilePath] -> FL (LabelledPatch p) wX wY -> [Label]
