@@ -49,7 +49,6 @@ module Darcs.Repository.Internal
     , finalizeRepositoryChanges
     , unrevertUrl
     , applyToWorking
-    , patchSetToPatches
     , createPristineDirectoryTree
     , createPartialsPristineDirectoryTree
     , reorderInventory
@@ -61,9 +60,12 @@ module Darcs.Repository.Internal
     , applyToTentativePristine
     , makeNewPending
     , seekRepo
+    , repoPatchType
+    , repoXor
     ) where
 
-import Prelude hiding ( catch )
+import Prelude ()
+import Darcs.Prelude
 
 import Darcs.Util.Printer ( putDocLn
                , (<+>)
@@ -74,12 +76,12 @@ import Darcs.Util.Printer ( putDocLn
                , ($$)
                )
 import Darcs.Util.Printer.Color (fancyPrinters)
-
+import Darcs.Util.Crypt.SHA1 ( SHA1, sha1Xor, zero )
 import Darcs.Repository.State ( readRecorded
                               , readWorking
                               , updateIndex
                               )
-import Darcs.Repository.LowLevel
+import Darcs.Repository.Pending
     ( readPending
     , readTentativePending
     , writeTentativePending
@@ -115,13 +117,13 @@ import Control.Monad ( when
                      , void
                      )
 
-import Control.Applicative ( (<$>) )
 import Control.Exception ( catch, IOException )
 
 import qualified Data.ByteString as B ( readFile
                                       , isPrefixOf
                                       )
 import qualified Data.ByteString.Char8 as BC (pack)
+import Data.List( foldl' )
 import Data.List.Ordered ( nubSort )
 import Data.Maybe ( fromMaybe )
 import Darcs.Patch ( Effect
@@ -133,11 +135,12 @@ import Darcs.Patch ( Effect
                    , commute
                    , fromPrim
                    , RepoPatch
+                   , IsRepoType
                    , Patchy
                    , merge
                    , listConflictedFiles
                    , listTouchedFiles
-                   , Named
+                   , WrappedNamed
                    , commuteRL
                    , fromPrims
                    , readPatch
@@ -163,13 +166,14 @@ import Darcs.Patch.Prim ( PrimPatchBase
 import Darcs.Patch.Bundle ( scanBundle
                           , makeBundleN
                           )
-import Darcs.Patch.Info ( isTag )
-import Darcs.Patch.MaybeInternal ( flIsInternal )
-import Darcs.Patch.Named ( patchcontents )
+import Darcs.Patch.Info ( isTag, makePatchname )
+import Darcs.Patch.Named.Wrapped ( namedIsInternal )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd
                                 , hopefully
                                 , info
                                 )
+import Darcs.Patch.Type ( PatchType(..) )
+
 import qualified Darcs.Repository.HashedRepo as HashedRepo
                             ( revertTentativeChanges
                             , finalizeTentativeChanges
@@ -178,6 +182,7 @@ import qualified Darcs.Repository.HashedRepo as HashedRepo
                             , copyPartialsPristine
                             , applyToTentativePristine
                             , addToTentativeInventory
+                            , readRepo
                             , readTentativeRepo
                             , readRepoUsingSpecificInventory
                             , cleanPristine
@@ -186,6 +191,7 @@ import qualified Darcs.Repository.HashedRepo as HashedRepo
                             )
 import qualified Darcs.Repository.Old as Old
                             ( revertTentativeChanges
+                            , readOldRepo
                             , oldRepoFailMsg
                             )
 import Darcs.Repository.Flags
@@ -223,7 +229,6 @@ import Darcs.Patch.Permutations ( commuteWhatWeCanFL
                                 )
 import Darcs.Patch.Set ( PatchSet(..)
                        , SealedPatchSet
-                       , newset2FL
                        , newset2RL
                        , Origin
                        )
@@ -252,7 +257,7 @@ import Darcs.Util.Workaround
     , setExecutable
     )
 import Darcs.Repository.Prefs ( getCaches )
-import Darcs.Repository.Lock
+import Darcs.Util.Lock
     ( writeDocBinFile
     , removeFileMayNotExist
     )
@@ -263,19 +268,19 @@ import Darcs.Util.Global ( darcsdir )
 
 import System.Mem( performGC )
 
-import qualified Storage.Hashed.Tree as Tree
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
+import qualified Darcs.Util.Tree as Tree
 import Darcs.Repository.PatchIndex ( createOrUpdatePatchIndexDisk, doesPatchIndexExist )
-import Darcs.Repository.Read ( readRepo )
 #include "impossible.h"
 
 -- | The status of a given directory: is it a darcs repository?
-data IdentifyRepo p wR wU wT = BadRepository String -- ^ looks like a repository with some error
-                             | NonRepository String -- ^ safest guess
-                             | GoodRepository (Repository p wR wU wT)
+data IdentifyRepo rt p wR wU wT
+    = BadRepository String -- ^ looks like a repository with some error
+    | NonRepository String -- ^ safest guess
+    | GoodRepository (Repository rt p wR wU wT)
 
 -- | Tries to identify the repository in a given directory
-maybeIdentifyRepository :: UseCache -> String -> IO (IdentifyRepo p wR wU wT)
+maybeIdentifyRepository :: UseCache -> String -> IO (IdentifyRepo rt p wR wU wT)
 maybeIdentifyRepository useCache "." =
     do darcs <- doesDirectoryExist darcsdir
        if not darcs
@@ -314,8 +319,8 @@ identifyPristine =
 
 -- | identifyRepository identifies the repo at 'url'. Warning:
 -- you have to know what kind of patches are found in that repo.
-identifyRepository :: forall p wR wU wT. UseCache -> String
-                           -> IO (Repository p wR wU wT)
+identifyRepository :: forall rt p wR wU wT. UseCache -> String
+                           -> IO (Repository rt p wR wU wT)
 identifyRepository useCache url =
     do er <- maybeIdentifyRepository useCache url
        case er of
@@ -325,11 +330,11 @@ identifyRepository useCache url =
 
 -- | @identifyRepositoryFor repo url@ identifies (and returns) the repo at 'url',
 -- but fails if it is not compatible for reading from and writing to.
-identifyRepositoryFor :: forall p wR wU wT vR vU vT. RepoPatch p
-                      => Repository p wR wU wT
+identifyRepositoryFor :: forall rt p wR wU wT vR vU vT. RepoPatch p
+                      => Repository rt p wR wU wT
                       -> UseCache
                       -> String
-                      -> IO (Repository p vR vU vT)
+                      -> IO (Repository rt p vR vU vT)
 identifyRepositoryFor (Repo _ source _ _) useCache url =
     do Repo absurl target x c <- identifyRepository useCache url
        case transferProblem target source of
@@ -357,6 +362,9 @@ amInHashedRepository wd
                        _ -> return (Left Old.oldRepoFailMsg)
        left    -> return left
 
+repoPatchType :: Repository rt p wR wU wT -> PatchType rt p
+repoPatchType _ = PatchType
+
 -- | hunt upwards for the darcs repository
 -- This keeps changing up one parent directory, testing at each
 -- step if the current directory is a repository or not.  $
@@ -409,7 +417,7 @@ findRepository _ = fromMaybe (Right ()) <$> seekRepo
 
 -- TODO: see also Repository.State.readPendingLL ... to be removed after GHC 7.2
 readNewPendingLL :: (RepoPatch p, ApplyState p ~ Tree)
-              => Repository p wR wU wT -> IO (Sealed ((FL p) wT))
+              => Repository rt p wR wU wT -> IO (Sealed ((FL p) wT))
 readNewPendingLL repo = mapSeal (mapFL_FL fromPrim) `fmap` readNewPending repo
 
 
@@ -417,8 +425,8 @@ readNewPendingLL repo = mapSeal (mapFL_FL fromPrim) `fmap` readNewPending repo
 --   @pendPs@ could be applied to pristine if we wanted to, and if so
 --   writes it to disk.  If it can't be applied, @pendPs@ must
 --   be somehow buggy, so we save it for forensics and crash.
-makeNewPending :: forall p wR wU wT wY. (RepoPatch p, ApplyState p ~ Tree)
-                 => Repository p wR wU wT
+makeNewPending :: forall rt p wR wU wT wY. (RepoPatch p, ApplyState p ~ Tree)
+                 => Repository rt p wR wU wT
                  -> UpdateWorking
                  -> FL (PrimOf p) wT wY
                  -> IO ()
@@ -470,37 +478,37 @@ siftForPending simple_ps =
     -- we can nuke the hunk, but not so in (hunk :> replace)
     sift :: FL prim wA wB -> RL prim wC wA -> Sealed (FL prim wC)
     sift sofar NilRL = seal sofar
-    sift sofar (p:<:ps) | primIsHunk p || primIsBinary p =
+    sift sofar (ps:<:p) | primIsHunk p || primIsBinary p =
         case commuteFLorComplain (p :> sofar) of
             Right (sofar' :> _) -> sift sofar'      ps
             Left _              -> sift (p:>:sofar) ps
-    sift sofar (p:<:ps) = sift (p:>:sofar) ps
+    sift sofar (ps:<:p) = sift (p:>:sofar) ps
 
-readTentativeRepo :: (RepoPatch p, ApplyState p ~ Tree)
-                  => Repository p wR wU wT
-                  -> IO (PatchSet p Origin wT)
+readTentativeRepo :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                  => Repository rt p wR wU wT
+                  -> IO (PatchSet rt p Origin wT)
 readTentativeRepo repo@(Repo r rf _ _)
     | formatHas HashedInventory rf = HashedRepo.readTentativeRepo repo r
     | otherwise = fail Old.oldRepoFailMsg
 
-readRepoUsingSpecificInventory :: (RepoPatch p, ApplyState p ~ Tree)
+readRepoUsingSpecificInventory :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
                                => String
-                               -> Repository p wR wU wT
-                               -> IO (PatchSet p Origin wT)
+                               -> Repository rt p wR wU wT
+                               -> IO (PatchSet rt p Origin wT)
 readRepoUsingSpecificInventory invPath repo@(Repo r rf _ _)
     | formatHas HashedInventory rf =
         HashedRepo.readRepoUsingSpecificInventory invPath repo r
     | otherwise = fail Old.oldRepoFailMsg
 
-prefsUrl :: Repository p wR wU wT -> String
+prefsUrl :: Repository rt p wR wU wT -> String
 prefsUrl (Repo r _ _ _) = r ++ "/"++darcsdir++"/prefs"
 
-unrevertUrl :: Repository p wR wU wT -> String
+unrevertUrl :: Repository rt p wR wU wT -> String
 unrevertUrl (Repo r _ _ _) = r ++ "/"++darcsdir++"/patches/unrevert"
 
 applyToWorking :: (ApplyState (PrimOf p) ~ Tree, RepoPatch p)
-               => Repository p wR wU wT -> Verbosity -> FL (PrimOf p) wU wY
-               -> IO (Repository p wR wY wT)
+               => Repository rt p wR wU wT -> Verbosity -> FL (PrimOf p) wU wY
+               -> IO (Repository rt p wR wY wT)
 applyToWorking (Repo r rf t c) verb patch =
   do
     unless (formatHas NoWorkingDir rf) $
@@ -515,10 +523,10 @@ applyToWorking (Repo r rf t c) verb patch =
 --   somewhere else.
 --
 --   Question (Eric Kow): how do we detect patch equivalence?
-tentativelyRemoveFromPending :: forall p wR wU wT wX wY. (RepoPatch p)
-                 => Repository p wR wU wT
+tentativelyRemoveFromPending :: forall rt p wR wU wT wX wY. (RepoPatch p)
+                 => Repository rt p wR wU wT
                  -> UpdateWorking
-                 -> PatchInfoAnd p wX wY
+                 -> PatchInfoAnd rt p wX wY
                  -> IO ()
 tentativelyRemoveFromPending _    NoUpdateWorking  _ = return ()
 tentativelyRemoveFromPending repo YesUpdateWorking p = do
@@ -527,7 +535,7 @@ tentativelyRemoveFromPending repo YesUpdateWorking p = do
     -- changepref patches in p? isSimple includes changepref, so what do
     -- adddir/etc have to do with it?  Why don't we we systematically
     -- crudeSift/not?
-    let effectp = if allFL isSimple pend
+    let effectp = if isSimple pend
                      then crudeSift $ effect p
                      else effect p
     Sealed newpend <- return $ rmpend (progressFL "Removing from pending:" effectp)
@@ -556,19 +564,38 @@ tentativelyRemoveFromPending repo YesUpdateWorking p = do
                     -- DJR: I don't think this last case should be
                     -- reached, but it also shouldn't lead to corruption.
 
-isSimple :: PrimPatch prim => prim wX wY -> Bool
-isSimple x = primIsHunk x || primIsBinary x || primIsSetpref x
-
--- This seems to do the opposite of sifting, ie. we retain hunk/binary patches
--- but delete changepref patches.
+-- | A sequence of primitive patches (candidates for the pending patch)
+--   is considered simple if we can reason about their continued status as
+--   pending patches solely on the basis of them being hunk/binary patches.
 --
--- Why not just filterOutFLFL (not . primIsSetpref)?  Is it important to only
--- have this behaviour when all other patches are either hunk or binary?
+--   Simple here seems to mean that all patches are either hunk/binary
+--   patches, or patches that cannot (indirectly) depend on hunk/binary
+--   patches.  For now, the only other kinds of patches in this category
+--   are changepref patches.
+--
+--   It might be tempting to add, say, adddir patches but it's probably not a
+--   good idea because Darcs also inverts patches a lot in its reasoning so an
+--   innocent addir may be inverted to a rmdir which in turn may depend on
+--   a rmfile, which in turn depends on a hunk/binary. Likewise, we would
+--   not want to add move patches to this category for similar reasons of
+--   a potential dependency chain forming.
+isSimple :: PrimPatch prim => FL prim wX wY -> Bool
+isSimple =
+    allFL isSimp
+  where
+    isSimp x = primIsHunk x || primIsBinary x || primIsSetpref x
+
+-- | 'crudeSift' can be seen as a first pass approximation of 'siftForPending'
+--    that works without having to do any commutation.  It either returns a
+--    sifted pending (if the input is simple enough for this crude approach)
+--    or has no effect.
 crudeSift :: forall prim wX wY . PrimPatch prim => FL prim wX wY -> FL prim wX wY
-crudeSift xs = if allFL isSimple xs then filterOutFLFL ishunkbinary xs else xs
-    where ishunkbinary :: prim wA wB -> EqCheck wA wB
-          ishunkbinary x | primIsHunk x || primIsBinary x = unsafeCoerceP IsEq
-                         | otherwise = NotEq
+crudeSift xs =
+    if isSimple xs then filterOutFLFL ishunkbinary xs else xs
+  where
+    ishunkbinary :: prim wA wB -> EqCheck wA wB
+    ishunkbinary x | primIsHunk x || primIsBinary x = unsafeCoerceP IsEq
+                   | otherwise = NotEq
 
 data HashedVsOld a = HvsO { old, hashed :: a }
 
@@ -602,16 +629,16 @@ announceMergeConflicts cmd allowConflicts externalMerge resolved_pw =
                           " "++cmd++" mark-conflicts\n"++
                           "to "++darcsdir++"/prefs/defaults in the target repo. "
 
-checkUnrecordedConflicts :: forall p wT wY. RepoPatch p
+checkUnrecordedConflicts :: forall rt p wT wY. RepoPatch p
                          => UpdateWorking
-                         -> FL (Named p) wT wY
+                         -> FL (WrappedNamed rt p) wT wY
                          -> IO Bool
 checkUnrecordedConflicts NoUpdateWorking _
  = return False -- because we are called by `darcs convert` hence we don't care
 checkUnrecordedConflicts _ pc =
     do repository <- identifyRepository NoUseCache "."
        cuc repository
-    where cuc :: Repository p wR wU wT -> IO Bool
+    where cuc :: Repository rt p wR wU wT -> IO Bool
           cuc r = do Sealed (mpend :: FL (PrimOf p) wT wX) <- readPending r :: IO (Sealed (FL (PrimOf p) wT))
                      case mpend of
                        NilFL -> return False
@@ -631,12 +658,12 @@ checkUnrecordedConflicts _ pc =
           fromPrims_ = fromPrims
 
 tentativelyAddPatch :: (RepoPatch p, ApplyState p ~ Tree)
-                    => Repository p wR wU wT
+                    => Repository rt p wR wU wT
                     -> Compression
                     -> Verbosity
                     -> UpdateWorking
-                    -> PatchInfoAnd p wT wY
-                    -> IO (Repository p wR wU wY)
+                    -> PatchInfoAnd rt p wT wY
+                    -> IO (Repository rt p wR wU wY)
 tentativelyAddPatch = tentativelyAddPatch_ UpdatePristine
 
 data UpdatePristine = UpdatePristine 
@@ -644,15 +671,15 @@ data UpdatePristine = UpdatePristine
                     | DontUpdatePristineNorRevert deriving Eq
 
 tentativelyAddPatches_
-                     :: forall p wR wU wT wY
+                     :: forall rt p wR wU wT wY
                       . (RepoPatch p, ApplyState p ~ Tree)
                      => UpdatePristine
-                     -> Repository p wR wU wT
+                     -> Repository rt p wR wU wT
                      -> Compression
                      -> Verbosity
                      -> UpdateWorking
-                     -> FL (PatchInfoAnd p) wT wY
-                     -> IO (Repository p wR wU wY)
+                     -> FL (PatchInfoAnd rt p) wT wY
+                     -> IO (Repository rt p wR wU wY)
 tentativelyAddPatches_ _up r _compr _verb _uw NilFL = return r
 tentativelyAddPatches_ up r compr verb uw (p:>:ps) = do
     r' <- tentativelyAddPatch_ up r compr verb uw p
@@ -660,15 +687,15 @@ tentativelyAddPatches_ up r compr verb uw (p:>:ps) = do
 
 -- TODO re-add a safety catch for --dry-run? Maybe using a global, like dryRun
 -- :: Bool, with dryRun = unsafePerformIO $ readIORef ...
-tentativelyAddPatch_ :: forall p wR wU wT wY
+tentativelyAddPatch_ :: forall rt p wR wU wT wY
                       . (RepoPatch p, ApplyState p ~ Tree)
                      => UpdatePristine
-                     -> Repository p wR wU wT
+                     -> Repository rt p wR wU wT
                      -> Compression
                      -> Verbosity
                      -> UpdateWorking
-                     -> PatchInfoAnd p wT wY
-                     -> IO (Repository p wR wU wY)
+                     -> PatchInfoAnd rt p wT wY
+                     -> IO (Repository rt p wR wU wY)
 
 tentativelyAddPatch_ up r@(Repo dir rf t c) compr verb uw p =
     withCurrentDirectory dir $ do
@@ -682,7 +709,7 @@ tentativelyAddPatch_ up r@(Repo dir rf t c) compr verb uw p =
        return (Repo dir rf t c)
 
 applyToTentativePristine :: (ApplyState q ~ Tree, Effect q, Patchy q, ShowPatch q, PrimPatchBase q)
-                         => Repository p wR wU wT
+                         => Repository rt p wR wU wT
                          -> Verbosity
                          -> q wT wY
                          -> IO ()
@@ -700,8 +727,8 @@ applyToTentativePristine (Repo dir rf _ _) verb p =
 --   This fuction is unsafe because it accepts a patch that works on the
 --   tentative pending and we don't currently track the state of the
 --   tentative pending.
-tentativelyAddToPending :: forall p wR wU wT wX wY. RepoPatch p
-                        => Repository p wR wU wT
+tentativelyAddToPending :: forall rt p wR wU wT wX wY. RepoPatch p
+                        => Repository rt p wR wU wT
                         -> UpdateWorking
                         -> FL (PrimOf p) wX wY
                         -> IO ()
@@ -719,8 +746,8 @@ tentativelyAddToPending repo@(Repo dir _ _ _) YesUpdateWorking patch =
 
 -- | setTentativePending is basically unsafe.  It overwrites the pending
 --   state with a new one, not related to the repository state.
-setTentativePending :: forall p wR wU wT wX wY. RepoPatch p
-                    => Repository p wR wU wT
+setTentativePending :: forall rt p wR wU wT wX wY. RepoPatch p
+                    => Repository rt p wR wU wT
                     -> UpdateWorking
                     -> FL (PrimOf p) wX wY
                     -> IO ()
@@ -735,8 +762,8 @@ setTentativePending repo@(Repo dir _ _ _) YesUpdateWorking patch = do
 --
 --   This function is basically unsafe.  It overwrites the pending state
 --   with a new one, not related to the repository state.
-prepend :: forall p wR wU wT wX wY. RepoPatch p
-        => Repository p wR wU wT
+prepend :: forall rt p wR wU wT wX wY. RepoPatch p
+        => Repository rt p wR wU wT
         -> UpdateWorking
         -> FL (PrimOf p) wX wY
         -> IO ()
@@ -750,21 +777,22 @@ prepend repo YesUpdateWorking patch = do
     newpend NilFL patch_ = seal patch_
     newpend p     patch_ = seal $ patch_ +>+ p
 
-tentativelyRemovePatches :: (RepoPatch p, ApplyState p ~ Tree)
-                         => Repository p wR wU wT
+tentativelyRemovePatches :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                         => Repository rt p wR wU wT
                          -> Compression
                          -> UpdateWorking
-                         -> FL (PatchInfoAnd p) wX wT
-                         -> IO (Repository p wR wU wX)
+                         -> FL (PatchInfoAnd rt p) wX wT
+                         -> IO (Repository rt p wR wU wX)
 tentativelyRemovePatches = tentativelyRemovePatches_ UpdatePristine
 
-tentativelyRemovePatches_ :: forall p wR wU wT wX. (RepoPatch p, ApplyState p ~ Tree)
+tentativelyRemovePatches_ :: forall rt p wR wU wT wX
+                           . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
                           => UpdatePristine
-                          -> Repository p wR wU wT
+                          -> Repository rt p wR wU wT
                           -> Compression
                           -> UpdateWorking
-                          -> FL (PatchInfoAnd p) wX wT
-                          -> IO (Repository p wR wU wX)
+                          -> FL (PatchInfoAnd rt p) wX wT
+                          -> IO (Repository rt p wR wU wX)
 tentativelyRemovePatches_ up repository@(Repo dir rf t c) compr uw ps =
     withCurrentDirectory dir $ do
       when (up == UpdatePristine) $ do debugMessage "Adding changes to pending..."
@@ -789,19 +817,20 @@ tentativelyRemovePatches_ up repository@(Repo dir rf t c) compr uw ps =
 -- with the same name and then adding the passed in sequence.
 -- Typically callers will have obtained the passed in sequence using
 -- 'findCommon' and friends.
-tentativelyReplacePatches :: forall p wR wU wT wX. (RepoPatch p, ApplyState p ~ Tree)
-                          => Repository p wR wU wT
+tentativelyReplacePatches :: forall rt p wR wU wT wX
+                           . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                          => Repository rt p wR wU wT
                           -> Compression
                           -> UpdateWorking
                           -> Verbosity
-                          -> FL (PatchInfoAnd p) wX wT
+                          -> FL (PatchInfoAnd rt p) wX wT
                           -> IO ()
 tentativelyReplacePatches repository compr uw verb ps =
-    do let ps' = filterOutFLFL (flIsInternal . patchcontents . hopefully) ps
+    do let ps' = filterOutFLFL (namedIsInternal . hopefully) ps
        repository' <- tentativelyRemovePatches_ DontUpdatePristineNorRevert repository compr uw ps'
        mapAdd repository' ps'
-  where mapAdd :: Repository p wM wL wI
-               -> FL (PatchInfoAnd p) wI wJ
+  where mapAdd :: Repository rt p wM wL wI
+               -> FL (PatchInfoAnd rt p) wI wJ
                -> IO ()
         mapAdd _ NilFL = return ()
         mapAdd r (a:>:as) =
@@ -817,7 +846,7 @@ tentativelyReplacePatches repository compr uw verb ps =
 --   inconsistency of the @NoUpdateWorking@ doing deletion, but
 --   @YesUpdateWorking@ not bothering.
 finalizePending :: (RepoPatch p, ApplyState p ~ Tree)
-                => Repository p wR wU wT
+                => Repository rt p wR wU wT
                 -> UpdateWorking
                 -> IO ()
 finalizePending (Repo dir _ _ _) NoUpdateWorking =
@@ -828,8 +857,8 @@ finalizePending repository@(Repo dir _ _ _) updateWorking@YesUpdateWorking =
       Sealed new_pending <- return $ siftForPending tpend
       makeNewPending repository updateWorking new_pending
 
-finalizeRepositoryChanges :: (RepoPatch p, ApplyState p ~ Tree)
-                          => Repository p wR wU wT
+finalizeRepositoryChanges :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                          => Repository rt p wR wU wT
                           -> UpdateWorking
                           -> Compression
                           -> IO ()
@@ -851,7 +880,7 @@ finalizeRepositoryChanges repository@(Repo dir rf _ _) updateWorking compr
 -- changes, revertRepositoryChanges also re-initialises the tentative state.
 -- It's therefore used before makign any changes to the repo.
 revertRepositoryChanges :: RepoPatch p
-                        => Repository p wR wU wT
+                        => Repository rt p wR wU wT
                         -> UpdateWorking
                         -> IO ()
 revertRepositoryChanges r@(Repo dir rf _ _) uw =
@@ -863,12 +892,10 @@ revertRepositoryChanges r@(Repo dir rf _ _) uw =
        decideHashedOrNormal rf HvsO { hashed = HashedRepo.revertTentativeChanges,
                                       old = Old.revertTentativeChanges }
 
-patchSetToPatches :: RepoPatch p => PatchSet p wX wY -> FL (Named p) wX wY
-patchSetToPatches patchSet = mapFL_FL hopefully $ newset2FL patchSet
-
-removeFromUnrevertContext :: forall p wR wU wT wX. (RepoPatch p, ApplyState p ~ Tree)
-                          => Repository p wR wU wT
-                          -> FL (PatchInfoAnd p) wX wT
+removeFromUnrevertContext :: forall rt p wR wU wT wX
+                           . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                          => Repository rt p wR wU wT
+                          -> FL (PatchInfoAnd rt p) wX wT
                           -> IO ()
 removeFromUnrevertContext repository ps = do
   Sealed bundle <- unrevert_patch_bundle `catchall` return (seal (PatchSet NilRL NilRL))
@@ -877,12 +904,12 @@ removeFromUnrevertContext repository ps = do
             do confirmed <- promptYorn "This operation will make unrevert impossible!\nProceed?"
                if confirmed then removeFileMayNotExist (unrevertUrl repository)
                             else fail "Cancelled."
-        unrevert_patch_bundle :: IO (SealedPatchSet p Origin)
+        unrevert_patch_bundle :: IO (SealedPatchSet rt p Origin)
         unrevert_patch_bundle = do pf <- B.readFile (unrevertUrl repository)
                                    case scanBundle pf of
                                      Right foo -> return foo
                                      Left err -> fail $ "Couldn't parse unrevert patch:\n" ++ err
-        remove_from_unrevert_context_ :: PatchSet p Origin wZ -> IO ()
+        remove_from_unrevert_context_ :: PatchSet rt p Origin wZ -> IO ()
         remove_from_unrevert_context_ (PatchSet NilRL NilRL) = return ()
         remove_from_unrevert_context_ bundle =
          do debugMessage "Adjusting the context of the unrevert changes..."
@@ -905,7 +932,7 @@ removeFromUnrevertContext repository ps = do
                              writeDocBinFile (unrevertUrl repository) bundle'
             debugMessage "Done adjusting the context of the unrevert changes!"
 
-cleanRepository :: RepoPatch p => Repository p wR wU wT -> IO ()
+cleanRepository :: RepoPatch p => Repository rt p wR wU wT -> IO ()
 cleanRepository repository@(Repo _ rf _ _) =
     decideHashedOrNormal rf
     HvsO { hashed = cleanHashedRepo repository,
@@ -919,7 +946,7 @@ cleanRepository repository@(Repo _ rf _ _) =
 
 -- | grab the pristine hash of _darcs/hash_inventory, and retrieve whole pristine tree,
 --   possibly writing a clean working copy in the process.
-createPristineDirectoryTree :: RepoPatch p => Repository p wR wU wT -> FilePath -> WithWorkingDir -> IO ()
+createPristineDirectoryTree :: RepoPatch p => Repository rt p wR wU wT -> FilePath -> WithWorkingDir -> IO ()
 createPristineDirectoryTree (Repo r rf _ c) reldir wwd
     | formatHas HashedInventory rf =
         do createDirectoryIfMissing True reldir
@@ -929,7 +956,7 @@ createPristineDirectoryTree (Repo r rf _ c) reldir wwd
 -- fp below really should be FileName
 -- | Used by the commands dist and diff
 createPartialsPristineDirectoryTree :: (FilePathLike fp, RepoPatch p)
-                                    => Repository p wR wU wT
+                                    => Repository rt p wR wU wT
                                     -> [fp]
                                     -> FilePath
                                     -> IO ()
@@ -941,7 +968,7 @@ createPartialsPristineDirectoryTree (Repo r rf _ c) prefs dir
     | otherwise = fail Old.oldRepoFailMsg
 
 withRecorded :: RepoPatch p
-             => Repository p wR wU wT
+             => Repository rt p wR wU wT
              -> ((AbsolutePath -> IO a) -> IO a)
              -> (AbsolutePath -> IO a)
              -> IO a
@@ -949,8 +976,8 @@ withRecorded repository mk_dir f
     = mk_dir $ \d -> do createPristineDirectoryTree repository (toFilePath d) WithWorkingDir
                         f d
 
-withTentative :: forall p a wR wU wT. (RepoPatch p, ApplyState p ~ Tree)
-              => Repository p wR wU wT
+withTentative :: forall rt p a wR wU wT. (RepoPatch p, ApplyState p ~ Tree)
+              => Repository rt p wR wU wT
               -> ((AbsolutePath -> IO a) -> IO a)
               -> (AbsolutePath -> IO a)
               -> IO a
@@ -1006,8 +1033,8 @@ setScriptsExecutablePatches = setScriptsExecutable_ . Just
 -- to a given tag are included in that tag, so less commutation and
 -- history traversal is needed.  This latter issue can become very
 -- important in large repositories.
-reorderInventory :: (RepoPatch p, ApplyState p ~ Tree)
-                 => Repository p wR wU wR
+reorderInventory :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                 => Repository rt p wR wU wR
                  -> Compression
                  -> UpdateWorking
                  -> Verbosity
@@ -1016,16 +1043,16 @@ reorderInventory repository@(Repo _ rf _ _) compr uw verb =
     decideHashedOrNormal rf HvsO {
     hashed = do
         debugMessage "Reordering the inventory."
-        PatchSet ps _ <- misplacedPatches `fmap` readRepo repository
+        PatchSet _ ps <- misplacedPatches `fmap` readRepo repository
         tentativelyReplacePatches repository compr uw verb $ reverseRL ps
         HashedRepo.finalizeTentativeChanges repository compr
         debugMessage "Done reordering the inventory.",
     old = fail Old.oldRepoFailMsg }
 
 -- | Returns the patches that make the most recent tag dirty.
-misplacedPatches :: forall p wS wX . RepoPatch p
-                 => PatchSet p wS wX
-                 -> PatchSet p wS wX
+misplacedPatches :: forall rt p wS wX . RepoPatch p
+                 => PatchSet rt p wS wX
+                 -> PatchSet rt p wS wX
 misplacedPatches ps = 
         -- Filter the repository keeping only with the tags, ordered from the
         -- most recent.
@@ -1036,7 +1063,30 @@ misplacedPatches ps =
                     -- the clean PatchSet "up to" the tag (ts), and a RL of
                     -- patches after the tag (r).
                     case splitOnTag lt ps of
-                        Just (PatchSet xs ts :> r) -> PatchSet (r+<+xs) ts
+                        Just (PatchSet ts xs :> r) -> PatchSet ts (xs+<+r)
                         _ -> impossible -- Because the tag is in ps.
 
+-- @todo: we should not have to open the result of HashedRepo and
+-- seal it.  Instead, update this function to work with type witnesses
+-- by fixing DarcsRepo to match HashedRepo in the handling of
+-- Repository state.
+readRepo :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+         => Repository rt p wR wU wT
+         -> IO (PatchSet rt p Origin wR)
+readRepo repo@(Repo r rf _ _)
+    | formatHas HashedInventory rf = HashedRepo.readRepo repo r
+    | otherwise = do Sealed ps <- Old.readOldRepo r
+                     return $ unsafeCoerceP ps
 
+-- | XOR of all hashes of the patches' metadata.
+-- It enables to quickly see whether two repositories
+-- have the same patches, independently of their order.
+-- It relies on the assumption that the same patch cannot
+-- be present twice in a repository.
+-- This checksum is not cryptographically secure,
+-- see http://robotics.stanford.edu/~xb/crypto06b/ .
+repoXor :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+        => Repository rt p wR wU wR -> IO SHA1
+repoXor repo = do
+  hashes <- mapRL (makePatchname . info) . newset2RL <$> readRepo repo
+  return $ foldl' sha1Xor zero hashes
