@@ -21,7 +21,8 @@
 module Darcs.Repository.Old ( readOldRepo,
                               revertTentativeChanges, oldRepoFailMsg ) where
 
-import Prelude hiding ( catch )
+import Prelude ()
+import Darcs.Prelude
 
 import Darcs.Util.Progress ( debugMessage, beginTedious, endTedious, finishedOneIO )
 import Darcs.Util.Path ( ioAbsoluteOrRemote, toPath )
@@ -34,19 +35,19 @@ import Darcs.Patch.PatchInfoAnd ( Hopefully, PatchInfoAnd,
 
 import qualified Data.ByteString.Char8 as BC (break, pack)
 
-import Darcs.Patch ( RepoPatch, Named,
+import Darcs.Patch ( RepoPatch, IsRepoType, WrappedNamed,
                      readPatch )
 
 import Darcs.Patch.Witnesses.Ordered ( RL(..) )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal, unseal, mapSeal )
 import Darcs.Patch.Info ( PatchInfo, makeFilename, readPatchInfos, showPatchInfo )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, Origin )
-import Darcs.Repository.External
+import Darcs.Util.External
     ( gzFetchFilePS
     , Cachable(..)
     , cloneFile
     )
-import Darcs.Repository.Lock ( writeBinFile )
+import Darcs.Util.Lock ( writeBinFile )
 import Darcs.Util.Printer ( renderString, RenderMode(..) )
 import Darcs.Util.Global ( darcsdir )
 
@@ -54,7 +55,7 @@ import Control.Exception ( catch, IOException )
 
 #include "impossible.h"
 
-readOldRepo :: RepoPatch p => String -> IO (SealedPatchSet p Origin)
+readOldRepo :: (IsRepoType rt, RepoPatch p) => String -> IO (SealedPatchSet rt p Origin)
 readOldRepo d = do
   realdir <- toPath `fmap` ioAbsoluteOrRemote d
   let k = "Reading inventory of repository "++d
@@ -63,7 +64,8 @@ readOldRepo d = do
                         (\e -> do hPutStrLn stderr ("Invalid repository:  " ++ realdir)
                                   ioError e)
 
-readRepoPrivate :: RepoPatch p => String -> FilePath -> FilePath -> IO (SealedPatchSet p Origin)
+readRepoPrivate :: (IsRepoType rt, RepoPatch p)
+                => String -> FilePath -> FilePath -> IO (SealedPatchSet rt p Origin)
 readRepoPrivate k d iname = do
     i <- gzFetchFilePS (d </> darcsdir </> iname) Uncachable
     finishedOneIO k iname
@@ -76,10 +78,10 @@ readRepoPrivate k d iname = do
                    _ -> (Nothing, reverse $ readPatchInfos i)
     Sealed ts <- unseal seal `fmap` unsafeInterleaveIO (read_ts parse mt)
     Sealed ps <- unseal seal `fmap` unsafeInterleaveIO (read_patches parse is)
-    return $ seal (PatchSet ps ts)
+    return $ seal (PatchSet ts ps)
     where read_ts :: RepoPatch p =>
-                     (forall wB . PatchInfo -> IO (Sealed (PatchInfoAnd p wB)))
-                  -> Maybe PatchInfo -> IO (Sealed (RL (Tagged p) Origin))
+                     (forall wB . PatchInfo -> IO (Sealed (PatchInfoAnd rt p wB)))
+                  -> Maybe PatchInfo -> IO (Sealed (RL (Tagged rt p) Origin))
           read_ts _ Nothing = do endTedious k
                                  return $ seal NilRL
           read_ts parse (Just tag0) =
@@ -100,22 +102,23 @@ readRepoPrivate k d iname = do
                                   \(e :: IOException) ->
                                         return $ seal $
                                         patchInfoAndPatch tag0 $ unavailable $ show e
-                 return $ seal $ Tagged tag00 Nothing ps :<: ts
-          parse2 :: RepoPatch p => PatchInfo -> FilePath
-                                -> IO (Sealed (PatchInfoAnd p wX))
+                 return $ seal $ ts :<: Tagged tag00 Nothing ps
+          parse2 :: (IsRepoType rt, RepoPatch p)
+                 => PatchInfo -> FilePath
+                 -> IO (Sealed (PatchInfoAnd rt p wX))
           parse2 i fn = do ps <- unsafeInterleaveIO $ gzFetchFilePS fn Cachable
                            return $ patchInfoAndPatch i
                              `mapSeal` hopefullyNoParseError (toPath fn) (readPatch ps)
-          hopefullyNoParseError :: String -> Maybe (Sealed (Named a1dr wX))
-                                -> Sealed (Hopefully (Named a1dr) wX)
+          hopefullyNoParseError :: String -> Maybe (Sealed (WrappedNamed rt a1dr wX))
+                                -> Sealed (Hopefully (WrappedNamed rt a1dr) wX)
           hopefullyNoParseError _ (Just (Sealed x)) = seal $ actually x
           hopefullyNoParseError s Nothing = seal $ unavailable $ "Couldn't parse file "++s
           read_patches :: RepoPatch p =>
-                          (forall wB . PatchInfo -> IO (Sealed (PatchInfoAnd p wB)))
-                       -> [PatchInfo] -> IO (Sealed (RL (PatchInfoAnd p) wX))
+                          (forall wB . PatchInfo -> IO (Sealed (PatchInfoAnd rt p wB)))
+                       -> [PatchInfo] -> IO (Sealed (RL (PatchInfoAnd rt p) wX))
           read_patches _ [] = return $ seal NilRL
           read_patches parse (i:is) =
-              lift2Sealed (:<:)
+              lift2Sealed (flip (:<:))
                           (read_patches parse is)
                           (parse i `catch` \(e :: IOException) ->
                            return $ seal $ patchInfoAndPatch i $ unavailable $ show e)
@@ -132,5 +135,5 @@ revertTentativeChanges =
        writeBinFile (darcsdir++"/tentative_pristine") ""
 
 oldRepoFailMsg :: String
-oldRepoFailMsg = "ERROR: repository upgrade required, try `darcs optimize --upgrade`\n"
+oldRepoFailMsg = "ERROR: repository upgrade required, try `darcs optimize upgrade`\n"
               ++ "See http://wiki.darcs.net/OF for more details."
