@@ -26,15 +26,13 @@ module Darcs.UI.Commands.Amend
     (
       amend
     , amendrecord
-    , updatePatchHeader
     ) where
 
-import Prelude hiding ( (^) )
+import Prelude ()
+import Darcs.Prelude
 
 import Data.Maybe ( isNothing, isJust )
-import Control.Applicative ( (<$>) )
-import Control.Monad ( unless, when )
-import System.Exit ( exitSuccess )
+import Control.Monad ( when )
 
 import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts
@@ -44,24 +42,21 @@ import Darcs.UI.Commands
     , setEnvDarcsPatches
     , amInHashedRepository
     )
-import Darcs.UI.Commands.Record ( getLog )
 import Darcs.UI.Commands.Util ( announceFiles, testTentativeAndMaybeExit )
-import Darcs.UI.Flags
-    ( DarcsFlag
-    , diffOpts, fixSubPaths, getEasyAuthor, promptAuthor, getDate )
+import Darcs.UI.Flags ( DarcsFlag, diffOpts, fixSubPaths )
 import Darcs.UI.Options ( DarcsOption, (^), oparse, odesc, ocheck, defaultFlags )
 import qualified Darcs.UI.Options.All as O
+import Darcs.UI.PatchHeader ( updatePatchHeader, AskAboutDeps(..)
+                            , HijackOptions(..)
+                            , runHijackT )
 import Darcs.Repository.Flags ( UpdateWorking(..), DryRun(NoDryRun) )
-import Darcs.Patch ( RepoPatch, description, PrimOf, fromPrims,
-                     infopatch, getdeps, adddeps, effect, invert, invertFL
+import Darcs.Patch ( IsRepoType, RepoPatch, description, PrimOf
+                   , effect, invert, invertFL
                    )
 import Darcs.Patch.Apply ( ApplyState )
-import Darcs.Patch.Info ( piAuthor, piName, piLog, piDateString,
-                          patchinfo, isInverted, isTag, invertName,
-                        )
-import Darcs.Patch.Prim ( canonizeFL )
+import Darcs.Patch.Info ( isTag )
 import Darcs.Patch.Split ( primSplitter )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia, hopefully, info, patchDesc )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, patchDesc )
 import Darcs.Patch.Rebase.Fixup ( RebaseFixup(..) )
 import Darcs.Patch.Rebase.Name ( RebaseName(..) )
 import Darcs.Util.Path ( toFilePath, SubPath(), AbsolutePath )
@@ -69,6 +64,7 @@ import Darcs.Repository
     ( Repository
     , withRepoLock
     , RepoJob(..)
+    , RebaseJobFlags(..)
     , tentativelyRemovePatches
     , tentativelyAddPatch
     , withManualRebaseUpdate
@@ -79,27 +75,24 @@ import Darcs.Repository
     , listRegisteredFiles
     )
 import Darcs.Repository.Prefs ( globalPrefsDirDoc )
-import Darcs.Repository.Util ( getMovesPs, getReplaces )
+import Darcs.Repository.State ( getMovesPs, getReplaces )
 import Darcs.UI.SelectChanges
-    ( selectChanges
-    , WhichChanges(..)
+    ( WhichChanges(..)
     , selectionContextPrim
     , runSelection
     , withSelectedPatchFromRepo
-    , askAboutDepends
     )
 import qualified Darcs.UI.SelectChanges as S
     ( PatchSelectionOptions(..)
     )
 import Darcs.Util.Exception ( clarifyErrors )
-import Darcs.Util.Prompt ( askUser )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), (:>)(..), (+>+), nullFL, reverseRL, mapFL_FL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 
 import Darcs.Util.Printer ( putDocLn )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 import Darcs.Repository.Internal ( tentativelyRemoveFromPending )
 
 
@@ -270,10 +263,11 @@ amendCmd fps cfg args = do
 
 doAmend :: AmendConfig -> Maybe [SubPath] -> IO ()
 doAmend cfg files =
+    let rebaseJobFlags = RebaseJobFlags (compress cfg) (verbosity cfg) YesUpdateWorking in
     withRepoLock NoDryRun (useCache cfg) YesUpdateWorking (umask cfg) $
-      RebaseAwareJob (compress cfg) (verbosity cfg) YesUpdateWorking $ \(repository :: Repository p wR wU wR) ->
+      RebaseAwareJob rebaseJobFlags $ \(repository :: Repository rt p wR wU wR) ->
     withSelectedPatchFromRepo "amend" repository (patchSelOpts cfg) $ \ (_ :> oldp) -> do
-        announceFiles files "Amending changes in"
+        announceFiles (verbosity cfg) files "Amending changes in"
             -- auxiliary function needed because the witness types differ for the isTag case
         pristine <- readRecorded repository
         let go :: forall wU1 . FL (PrimOf p) wR wU1 -> IO ()
@@ -282,22 +276,21 @@ doAmend cfg files =
               do let context = selectionContextPrim First "record"
                                       (patchSelOpts cfg)
                                       --([All,Unified] `intersect` opts)
-                                      (Just primSplitter)
+                                      (Just (primSplitter (diffAlgorithm cfg)))
                                       (map toFilePath <$> files)
                                       (Just pristine)
-                 (chosenPatches :> _) <- runSelection (selectChanges ch) context
+                 (chosenPatches :> _) <- runSelection ch context
                  addChangesToPatch cfg repository oldp chosenPatches
         if not (isTag (info oldp))
               -- amending a normal patch
            then if amendUnrecord cfg
-                   then do let sel = selectChanges (effect oldp)
-                               context = selectionContextPrim Last "unrecord"
+                   then do let context = selectionContextPrim Last "unrecord"
                                              (patchSelOpts cfg)
                                              -- ([All,Unified] `intersect` opts)
-                                             (Just primSplitter)
+                                             (Just (primSplitter (diffAlgorithm cfg)))
                                              (map toFilePath <$> files)
                                              (Just pristine)
-                           (_ :> chosenPrims) <- runSelection sel context
+                           (_ :> chosenPrims) <- runSelection (effect oldp) context
                            let invPrims = reverseRL (invertFL chosenPrims)
                            addChangesToPatch cfg repository oldp invPrims
                    else do Sealed replacePs <- if O.replaces (lookfor cfg) == O.YesLookForReplaces
@@ -306,8 +299,9 @@ doAmend cfg files =
                            movesPs <- if O.moves (lookfor cfg) == O.YesLookForMoves
                               then getMovesPs repository files
                               else return NilFL
-                           go =<< unrecordedChangesWithPatches (diffingOpts cfg) repository files
-                                                               movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
+                           go =<< unrecordedChangesWithPatches
+                                    movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
+                                    (diffingOpts cfg) repository files
               -- amending a tag
            else if hasEditMetadata cfg && isNothing files
                         -- the user is not trying to add new changes to the tag so there is
@@ -323,13 +317,15 @@ doAmend cfg files =
                            go NilFL
 
 
-addChangesToPatch :: forall p wR wU wT wX wY . (RepoPatch p, ApplyState p ~ Tree)
+addChangesToPatch :: forall rt p wR wU wT wX wY
+                   . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
                   => AmendConfig
-                  -> Repository p wR wU wT
-                  -> PatchInfoAnd p wX wT
+                  -> Repository rt p wR wU wT
+                  -> PatchInfoAnd rt p wX wT
                   -> FL (PrimOf p) wT wY
                   -> IO ()
 addChangesToPatch cfg repository oldp chs =
+    let rebaseJobFlags = RebaseJobFlags (compress cfg) (verbosity cfg) YesUpdateWorking in
     if nullFL chs && not (hasEditMetadata cfg)
     then putStrLn "You don't want to record anything!"
     else do
@@ -344,18 +340,19 @@ addChangesToPatch cfg repository oldp chs =
          --
          -- We can also signal that any explicit dependencies of the old patch should be rewritten
          -- for the new patch using a 'NameFixup'.
-         (repository''', (mlogf, newp)) <- withManualRebaseUpdate (compress cfg) (verbosity cfg) YesUpdateWorking repository $ \repository' -> do
+         (repository''', (mlogf, newp)) <- withManualRebaseUpdate rebaseJobFlags repository $ \repository' -> do
 
              repository'' <- tentativelyRemovePatches repository' (compress cfg) YesUpdateWorking (oldp :>: NilFL)
-             (mlogf, newp) <- updatePatchHeader
-                  (askDeps cfg)
+             (mlogf, newp) <- runHijackT AlwaysRequestHijackPermission $ updatePatchHeader "amend"
+                  (if askDeps cfg then AskAboutDeps repository'' else NoAskAboutDeps)
                   (patchSelOpts cfg)
+                  (diffAlgorithm cfg)
                   (keepDate cfg)
                   (selectAuthor cfg)
                   (author cfg)
                   (patchname cfg)
                   (askLongComment cfg)
-                  repository'' oldp chs
+                  oldp chs
              let fixups =
                    mapFL_FL PrimFixup (invert chs) +>+
                    NameFixup (Rename (info newp) (info oldp)) :>:
@@ -364,7 +361,7 @@ addChangesToPatch cfg repository oldp chs =
              repository''' <- tentativelyAddPatch repository'' (compress cfg) (verbosity cfg) YesUpdateWorking newp
              return (repository''', fixups, (mlogf, newp))
          let failmsg = maybe "" (\lf -> "\nLogfile left in "++lf++".") mlogf
-         testTentativeAndMaybeExit repository''' (verbosity cfg) (testChanges cfg) (sse cfg) (isInteractive True cfg)
+         testTentativeAndMaybeExit repository''' (verbosity cfg) (testChanges cfg) (sse cfg) (isInteractive cfg)
               ("you have a bad patch: '" ++ patchDesc newp ++ "'") "amend it"
               (Just failmsg)
          when (O.moves (lookfor cfg) == O.YesLookForMoves || O.replaces (lookfor cfg) == O.YesLookForReplaces)
@@ -375,56 +372,6 @@ addChangesToPatch cfg repository oldp chs =
          setEnvDarcsPatches (newp :>: NilFL)
 
 
-updatePatchHeader :: forall p wX wY wR wU wT . (RepoPatch p, ApplyState p ~ Tree)
-                  => Bool -- askDeps
-                  -> S.PatchSelectionOptions
-                  -> Bool -- keepDate
-                  -> Bool -- selectAuthor
-                  -> Maybe String -- author
-                  -> Maybe String -- patchname
-                  -> Maybe O.AskLongComment
-                  -> Repository p wR wU wT
-                  -> PatchInfoAnd p wT wX
-                  -> FL (PrimOf p) wX wY
-                  -> IO (Maybe String, PatchInfoAnd p wT wY)
-updatePatchHeader ask_deps pSelOpts nKeepDate nSelectAuthor nAuthor nPatchname nAskLongComment repository oldp chs = do
-
-    let newchs = canonizeFL (S.diffAlgorithm pSelOpts) (effect oldp +>+ chs)
-
-    let old_pdeps = getdeps $ hopefully oldp
-    newdeps <- if ask_deps
-               then askAboutDepends repository newchs pSelOpts old_pdeps
-               else return old_pdeps
-
-    let old_pinf = info oldp
-        prior    = (piName old_pinf, piLog old_pinf)
-        old_author = piAuthor old_pinf
-    date <- if nKeepDate then return (piDateString old_pinf) else getDate False
-    (new_author,edit_author) <- getAuthor nSelectAuthor nAuthor old_author
-    warnIfHijacking old_author edit_author
-    (new_name, new_log, mlogf) <- getLog 
-        nPatchname False (O.Logfile Nothing False) nAskLongComment (Just prior) chs
-    let maybe_invert = if isInverted old_pinf then invertName else id
-    new_pinf <- maybe_invert `fmap` patchinfo date new_name
-                                              new_author new_log
-
-    let newp = n2pia (adddeps (infopatch new_pinf (fromPrims newchs)) newdeps)
-
-    return (mlogf, newp)
-
-
-warnIfHijacking :: String -- Original author
-                -> Bool   -- Author change requested by options
-                -> IO ()
-warnIfHijacking old_author edit_author = do
-    authors_here <- getEasyAuthor
-    unless (edit_author || old_author `elem` authors_here) $ do
-      yorn <- askUser $
-          "You're not " ++ old_author ++"! Amend anyway? "
-      case yorn of ('y':_) -> return ()
-                   _       -> exitSuccess
-
-
 hasEditMetadata :: AmendConfig -> Bool
 hasEditMetadata cfg = isJust (author cfg)
                     || selectAuthor cfg
@@ -444,33 +391,18 @@ hasEditMetadata cfg = isJust (author cfg)
 -- hasEditMetadata (_:fs)                = hasEditMetadata fs
 
 
-getAuthor :: Bool -> Maybe String -> String -> IO (String,Bool)
-getAuthor True _ _ = do
-  a <- promptAuthor False True
-  return (a,True)
-getAuthor False (Just a) _ = return (a,True)
-getAuthor False Nothing old = return (old,False)
-
--- getAuthor (SelectAuthor:_) _  = do
---   a <- promptAuthor False True
---   return (a,True)
--- getAuthor (Author a:_) _  = return (a,True)
--- getAuthor (_:as) old      = getAuthor as old
--- getAuthor [] old          = return (old,False)
-
 patchSelOpts :: AmendConfig -> S.PatchSelectionOptions
 patchSelOpts cfg = S.PatchSelectionOptions
     { S.verbosity = verbosity cfg
     , S.matchFlags = matchFlags cfg
-    , S.diffAlgorithm = diffAlgorithm cfg
-    , S.interactive = isInteractive True cfg
+    , S.interactive = isInteractive cfg
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.summary = O.NoSummary -- option not supported, use default
     , S.withContext = withContext cfg
     }
 
 diffingOpts :: AmendConfig -> (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
-diffingOpts cfg = diffOpts (useIndex cfg) (O.adds (lookfor cfg)) False (diffAlgorithm cfg)
+diffingOpts cfg = diffOpts (useIndex cfg) (O.adds (lookfor cfg)) O.NoIncludeBoring (diffAlgorithm cfg)
 
-isInteractive :: Bool -> AmendConfig -> Bool
-isInteractive def = maybe def id . interactive
+isInteractive :: AmendConfig -> Bool
+isInteractive = maybe True id . interactive
