@@ -359,8 +359,7 @@ manpageLines = [
 
 markdownLines :: [String]
 markdownLines =
- [ "Darcs " ++ version, ""
- , "# Commands", ""
+ [ "# Commands", ""
  , unlines commands
  , "# Environment variables"
  , "", unlines environment
