@@ -19,10 +19,11 @@
 
 module Darcs.UI.Commands.Optimize ( optimize, doOptimizeHTTP ) where
 
-import Prelude hiding ( (^) )
-import Control.Applicative ( (<$>) )
+import Prelude ()
+import Darcs.Prelude
+
 import Control.Exception ( finally )
-import Control.Monad ( when, unless, forM_ )
+import Control.Monad ( when, unless, forM, forM_ )
 import Data.Maybe ( isJust, fromJust )
 import Data.List ( sort )
 import Data.Set ( difference )
@@ -42,7 +43,8 @@ import qualified Data.ByteString.Lazy as BL
 
 
 import Darcs.Patch.PatchInfoAnd ( extractHash )
-import Darcs.UI.Commands ( DarcsCommand(..), nodefaults, amInRepository, putInfo
+import Darcs.UI.Commands ( DarcsCommand(..), nodefaults
+                         , amInHashedRepository, amInRepository, putInfo
                          , normalCommand, withStdOpts )
 import Darcs.Repository.Prefs ( getPreflist, getCaches, globalCacheDir, oldGlobalCacheDir )
 import Darcs.Repository
@@ -53,21 +55,22 @@ import Darcs.Repository
     , reorderInventory
     , cleanRepository
     , replacePristine
+    , maybeIdentifyRepository
     )
+import Darcs.Repository.Internal ( IdentifyRepo(..) )
 import Darcs.Repository.HashedRepo ( inventoriesDir, patchesDir, pristineDir,
                                     hashedInventory, filterDirContents,
                                     readHashedPristineRoot, listInventoriesRepoDir,
                                     listPatchesLocalBucketed, set, unset, inv2pris )
 import Darcs.Repository.HashedIO ( getHashedFiles )
 import Darcs.Repository.Old ( oldRepoFailMsg )
-import Darcs.Repository.InternalTypes ( Repository(..) )
-import Darcs.Repository.Util ( getRecursiveDarcsRepos )
+import Darcs.Repository.InternalTypes ( Repository(..), Pristine(..) )
 import Darcs.Patch.Witnesses.Ordered
      ( mapFL
      , bunchFL
      , lengthRL
      )
-import Darcs.Patch ( RepoPatch )
+import Darcs.Patch ( IsRepoType, RepoPatch )
 import Darcs.Patch.Set
     ( newset2RL
     , newset2FL
@@ -76,7 +79,7 @@ import Darcs.Patch.Set
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Util.ByteString ( gzReadFilePS )
 import Darcs.Util.Printer ( text )
-import Darcs.Repository.Lock
+import Darcs.Util.Lock
     ( maybeRelink
     , gzWriteAtomicFilePS
     , writeAtomicFilePS
@@ -110,7 +113,7 @@ import Darcs.UI.Options ( DarcsOption, (^), oid, odesc, ocheck, onormalise
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags
     ( UpdateWorking (..), DryRun ( NoDryRun ), UseCache (..), UMask (..)
-    , WithWorkingDir(WithWorkingDir) )
+    , WithWorkingDir(WithWorkingDir), PatchFormat(PatchFormat1) )
 import Darcs.Patch.Progress ( progressFL )
 import Darcs.Repository.Cache ( hashedDir, bucketFolder,
                                 HashedDir(HashedPristineDir) )
@@ -125,7 +128,7 @@ import Darcs.Repository.PatchIndex
 import qualified Darcs.Repository.HashedRepo as HashedRepo
 import Darcs.Repository.State ( readRecorded )
 
-import Storage.Hashed.Tree
+import Darcs.Util.Tree
     ( Tree
     , TreeItem(..)
     , list
@@ -133,8 +136,8 @@ import Storage.Hashed.Tree
     , emptyTree
     )
 import Darcs.Util.Path( anchorPath, toFilePath, AbsolutePath )
-import Storage.Hashed.Plain( readPlainTree )
-import Storage.Hashed.Darcs
+import Darcs.Util.Tree.Plain( readPlainTree )
+import Darcs.Util.Tree.Hashed
     ( writeDarcsHashed
     , decodeDarcsSize
     )
@@ -197,7 +200,7 @@ common = DarcsCommand
     { commandProgramName = "darcs"
     , commandExtraArgs = 0
     , commandExtraArgHelp = []
-    , commandPrereq =  amInRepository
+    , commandPrereq =  amInHashedRepository
     , commandArgdefaults = nodefaults
     , commandName = undefined
     , commandHelp = undefined
@@ -221,7 +224,7 @@ optimizeClean = common
     }
 
 optimizeCleanCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeCleanCmd _ opts _ = do
+optimizeCleanCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       putInfo opts "Done cleaning repository!"
@@ -231,6 +234,7 @@ optimizeUpgrade = common
     { commandName = "upgrade"
     , commandHelp = "Convert old-fashioned repositories to the current default hashed format."
     , commandDescription = "upgrade repository to latest compatible format"
+    , commandPrereq = amInRepository
     , commandCommand = optimizeUpgradeCmd
     }
 
@@ -243,7 +247,7 @@ optimizeHttp = common
     }
 
 optimizeHttpCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeHttpCmd _ opts _ = do
+optimizeHttpCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       doOptimizeHTTP repository
@@ -259,7 +263,7 @@ optimizePristine = common
     }
 
 optimizePristineCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizePristineCmd _ opts _ = do
+optimizePristineCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       doOptimizePristine opts repository
@@ -282,14 +286,14 @@ optimizeUncompress = common
     }
 
 optimizeCompressCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeCompressCmd _ opts _ = do
+optimizeCompressCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       optimizeCompression [Compress]
       putInfo opts "Done optimizing by compression!"
 
 optimizeUncompressCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeUncompressCmd _ opts _ = do
+optimizeUncompressCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       optimizeCompression []
@@ -331,13 +335,13 @@ optimizeDisablePatchIndex = common
     }
 
 optimizeEnablePatchIndexCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeEnablePatchIndexCmd _ opts _ = do
+optimizeEnablePatchIndexCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       createOrUpdatePatchIndexDisk repository
       putInfo opts "Done enabling patch index!"
 
 optimizeDisablePatchIndexCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeDisablePatchIndexCmd _ opts _ = do
+optimizeDisablePatchIndexCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \(Repo repodir _ _ _) -> do
       deletePatchIndex repodir
       putInfo opts "Done disabling patch index!"
@@ -354,7 +358,7 @@ optimizeReorder = common
     }
 
 optimizeReorderCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeReorderCmd _ opts _ = do
+optimizeReorderCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       reorderInventory repository (compression opts) YesUpdateWorking (verbosity opts)
       putInfo opts "Done reordering!"
@@ -393,7 +397,7 @@ optimizeRelink = common
     }
 
 optimizeRelinkCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-optimizeRelinkCmd _ opts _ = do
+optimizeRelinkCmd _ opts _ =
     withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       doRelink opts
@@ -430,10 +434,8 @@ optimizeHelpRelink =
  "repository, or if you pulled the same patch from a remote repository\n" ++
  "into multiple local repositories."
 
-doOptimizePristine :: (RepoPatch p, ApplyState p ~ Tree) => [DarcsFlag] -> Repository p wR wU wT -> IO ()
+doOptimizePristine :: (RepoPatch p, ApplyState p ~ Tree) => [DarcsFlag] -> Repository rt p wR wU wT -> IO ()
 doOptimizePristine opts repo = do
-  hashed <- doesFileExist $ darcsdir </> "hashed_inventory"
-  when hashed $ do
     inv <- BS.readFile (darcsdir </> "hashed_inventory")
     let linesInv = BS.split '\n' inv
     case linesInv of
@@ -468,6 +470,7 @@ maybeRelinkFile (h:t) f =
        unless done $
            maybeRelinkFile t f
 
+-- Only 'optimize' commands that works on old-fashionned repositories
 optimizeUpgradeCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeUpgradeCmd _ opts _ = do
   debugMessage "Upgrading to hashed..."
@@ -479,7 +482,9 @@ optimizeUpgradeCmd _ opts _ = do
              withRepoLock NoDryRun YesUseCache YesUpdateWorking NoUMask $ RepoJob $ \repository ->
                actuallyUpgradeFormat repository
 
-actuallyUpgradeFormat :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO ()
+actuallyUpgradeFormat
+  :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+  => Repository rt p wR wU wT -> IO ()
 actuallyUpgradeFormat repository = do
   -- convert patches/inventory
   patches <- readRepo repository
@@ -500,7 +505,7 @@ actuallyUpgradeFormat repository = do
   sequence_ $ mapFL HashedRepo.applyToTentativePristine $ bunchFL 100 patchesToApply
   -- now make it official
   HashedRepo.finalizeTentativeChanges repository compr
-  writeRepoFormat (createRepoFormat True WithWorkingDir) (darcsdir </> "format")
+  writeRepoFormat (createRepoFormat PatchFormat1 WithWorkingDir) (darcsdir </> "format")
   -- clean out old-fashioned junk
   debugMessage "Cleaning out old-fashioned repository files..."
   removeFile   $ darcsdir </> "inventory"
@@ -517,7 +522,9 @@ actuallyUpgradeFormat repository = do
       gzs <- filter ((== ".gz") . takeExtension) `fmap` getDirectoryContents "."
       mapM_ removeFile gzs
 
-doOptimizeHTTP :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO ()
+doOptimizeHTTP
+  :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+  => Repository rt p wR wU wT -> IO ()
 doOptimizeHTTP repo = flip finally (mapM_ removeFileIfExists
   [ darcsdir </> "meta-filelist-inventories"
   , darcsdir </> "meta-filelist-pristine"
@@ -525,6 +532,7 @@ doOptimizeHTTP repo = flip finally (mapM_ removeFileIfExists
   , patchesTar <.> "part"
   ]) $ do
   rf <- identifyRepoFormat "."
+  -- function is exposed in API so could be called on non-hashed repo
   unless (formatHas HashedInventory rf) $ fail oldRepoFailMsg
   createDirectoryIfMissing False packsDir
   -- pristine hash
@@ -535,6 +543,9 @@ doOptimizeHTTP repo = flip finally (mapM_ removeFileIfExists
   is <- map ((darcsdir </> "inventories") </>) <$> HashedRepo.listInventories
   writeFile (darcsdir </> "meta-filelist-inventories") . unlines $
     map takeFileName is
+  -- tinkering with zlib's compression parameters does not make
+  -- any noticeable difference in generated archive size.
+  -- switching to bzip2 does OTOH (~25% size gain).
   BL.writeFile (patchesTar <.> "part") . compress . write =<<
     mapM fileEntry' ((darcsdir </> "meta-filelist-inventories") : ps ++
     reverse is)
@@ -557,7 +568,7 @@ doOptimizeHTTP repo = flip finally (mapM_ removeFileIfExists
     tp <- either fail return $ toTarPath False x
     return $ fileEntry tp content
   dirContents d = map ((darcsdir </> d) </>) <$>
-    (filterDirContents d $ const True)
+    filterDirContents d (const True)
   hashedPatchFileName x = case extractHash x of
     Left _ -> fail "unexpected unhashed patch"
     Right h -> darcsdir </> "patches" </> h
@@ -672,7 +683,7 @@ cleanGlobalCache dirs opts = do
   createDirectoryIfMissing True gCachePristineDir
 
   remove listInventoriesRepoDir gCacheInvDir repoPaths
-  remove ((listPatchesLocalBucketed gCache) . (</> darcsdir)) gCachePatchesDir repoPaths
+  remove (listPatchesLocalBucketed gCache . (</> darcsdir)) gCachePatchesDir repoPaths
   remove getPristine gCachePristineDir repoPaths
 
   where
@@ -682,13 +693,36 @@ cleanGlobalCache dirs opts = do
     remove' cacheSubDir s2 (concat s1)
 
   remove' :: String -> [String] -> [String] -> IO ()
-  remove' dir s1 s2 = do
+  remove' dir s1 s2 =
     mapM_ (removeFileMayNotExist . (\hashedFile ->
       dir </> bucketFolder hashedFile </> hashedFile))
-      (unset $ (set s1) `difference` (set s2))
+      (unset $ set s1 `difference` set s2)
 
   getPristine :: String -> IO [String]
   getPristine darcsDir = do
     i <- gzReadFilePS (darcsDir </> darcsdir </> hashedInventory)
-    priss <- getHashedFiles (darcsDir </> darcsdir </> pristineDir) [inv2pris i]
-    return priss
+    getHashedFiles (darcsDir </> darcsdir </> pristineDir) [inv2pris i]
+
+-- |getRecursiveDarcsRepos returns all paths to repositories under topdir.
+getRecursiveDarcsRepos :: FilePath -> IO [FilePath]
+getRecursiveDarcsRepos topdir = do
+  isDir <- doesDirectoryExist topdir
+  if isDir
+    then do
+      status <- maybeIdentifyRepository NoUseCache topdir
+      case status of
+        GoodRepository (Repo _ _ pris _)  ->
+                                case pris of
+                                  HashedPristine -> return [topdir]
+                                  _ -> return [] -- old fashioned or broken repo
+        _                 -> getRecursiveDarcsRepos' topdir
+    else return []
+
+  where
+    getRecursiveDarcsRepos' d = do
+      names <- getDirectoryContents d
+      let properNames = filter (\x -> head x /= '.') names
+      paths <- forM properNames $ \name -> do
+        let path = d </> name
+        getRecursiveDarcsRepos path
+      return (concat paths)
