@@ -20,11 +20,14 @@
 
 module Darcs.UI.Commands.Unrevert ( unrevert, writeUnrevert ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( (^), catch )
 
 import Control.Exception ( catch, IOException )
 import System.Exit ( exitSuccess )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
 import Darcs.UI.Flags
@@ -42,20 +45,20 @@ import Darcs.Repository ( SealedPatchSet, Repository, withRepoLock, RepoJob(..),
                           readRepo,
                           readRecorded,
                           applyToWorking, unrecordedChanges )
-import Darcs.Patch ( RepoPatch, PrimOf, commute, namepatch, fromPrims )
+import Darcs.Patch ( IsRepoType, RepoPatch, PrimOf, commute, fromPrims )
 import Darcs.Patch.Apply( ApplyState )
+import Darcs.Patch.Named.Wrapped ( namepatch )
 import Darcs.Patch.Set ( Origin )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed) )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), (+>+) )
 import Darcs.UI.SelectChanges
-    ( selectChanges
-    , WhichChanges(First)
+    ( WhichChanges(First)
     , runSelection
     , selectionContextPrim
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import qualified Data.ByteString as B
-import Darcs.Repository.Lock ( writeDocBinFile, removeFileMayNotExist )
+import Darcs.Util.Lock ( writeDocBinFile, removeFileMayNotExist )
 import Darcs.Patch.Depends ( mergeThem )
 import Darcs.UI.External ( catchall )
 import Darcs.Util.Prompt ( askUser )
@@ -68,7 +71,7 @@ import Darcs.Util.Path ( AbsolutePath )
 
 unrevertDescription :: String
 unrevertDescription =
- "Undo the last revert (may fail if changes after the revert)."
+ "Undo the last revert."
 
 unrevertHelp :: String
 unrevertHelp =
@@ -121,7 +124,6 @@ patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = []
-    , S.diffAlgorithm = diffAlgorithm flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.summary = O.NoSummary -- option not supported, use default
@@ -162,7 +164,7 @@ unrevertCmd _ opts [] =
                       ( UseIndex, ScanKnown, diffAlgorithm opts )
                       NilFL h_them
   let context = selectionContextPrim First "unrevert" (patchSelOpts opts) Nothing Nothing (Just recorded)
-  (p :> skipped) <- runSelection (selectChanges pw) context
+  (p :> skipped) <- runSelection pw context
   tentativelyAddToPending repository YesUpdateWorking p
   withSignalsBlocked $
       do finalizeRepositoryChanges repository YesUpdateWorking (compression opts)
@@ -174,8 +176,8 @@ unrevertCmd _ opts [] =
   debugMessage "Finished unreverting."
 unrevertCmd _ _ _ = impossible
 
-writeUnrevert :: (RepoPatch p, ApplyState p ~ Tree)
-              => Repository p wR wU wT -> FL (PrimOf p) wX wY
+writeUnrevert :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+              => Repository rt p wR wU wT -> FL (PrimOf p) wX wY
               -> Tree IO -> FL (PrimOf p) wR wX -> IO ()
 writeUnrevert repository NilFL _ _ = removeFileMayNotExist $ unrevertUrl repository
 writeUnrevert repository ps recorded pend =
@@ -190,10 +192,10 @@ writeUnrevert repository ps recorded pend =
         np <- namepatch date "unrevert" "anon" [] (fromRepoPrims repository p')
         bundle <- makeBundleN (Just recorded) rep (np :>: NilFL)
         writeDocBinFile (unrevertUrl repository) bundle
-        where fromRepoPrims :: RepoPatch p => Repository p wR wU wT -> FL (PrimOf p) wR wY -> FL p wR wY
+        where fromRepoPrims :: RepoPatch p => Repository rt p wR wU wT -> FL (PrimOf p) wR wY -> FL p wR wY
               fromRepoPrims _ = fromPrims
 
-unrevertPatchBundle :: RepoPatch p => Repository p wR wU wT -> IO (SealedPatchSet p Origin)
+unrevertPatchBundle :: RepoPatch p => Repository rt p wR wU wT -> IO (SealedPatchSet rt p Origin)
 unrevertPatchBundle repository = do
   pf <- B.readFile (unrevertUrl repository)
         `catchall` fail "There's nothing to unrevert!"
