@@ -16,9 +16,10 @@
 --  Boston, MA 02110-1301, USA.
 
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.UI.Commands.Util
     ( announceFiles
-    , filterExistingFiles
+    , filterExistingPaths
     , testTentativeAndMaybeExit
     , getUniqueRepositoryName
     , getUniqueDPatchName
@@ -26,34 +27,37 @@ module Darcs.UI.Commands.Util
 
 import Control.Monad ( unless )
 
+import Prelude ()
+import Darcs.Prelude
+
 import System.Exit ( ExitCode(..), exitWith )
 
-import Storage.Hashed.Monad ( virtualTreeIO, exists )
-import Storage.Hashed.Tree ( Tree )
-import Storage.Hashed( floatPath, readPlainTree )
+import Darcs.Util.Tree.Monad ( virtualTreeIO, exists )
+import Darcs.Util.Tree ( Tree )
 
-import Darcs.Util.Path
-    ( SubPath, toFilePath, getUniquePathName )
 import Darcs.Patch ( RepoPatch )
-import Darcs.Repository ( Repository, readRecorded, readUnrecorded,
-                          testTentative )
-import Darcs.Repository.State ( applyTreeFilter, restrictBoring )
-import Darcs.Repository.Flags ( LookForAdds (..) )
+import Darcs.Repository ( Repository, readRecorded, testTentative )
+import Darcs.Repository.State ( readUnrecordedFiltered )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Bundle ( patchFilename )
 import Darcs.UI.Options.All
-    ( Verbosity, SetScriptsExecutable, TestChanges (..)
-    , RunTest(..), LeaveTestDir(..) )
+    ( Verbosity(..), SetScriptsExecutable, TestChanges (..)
+    , RunTest(..), LeaveTestDir(..), UseIndex, ScanKnown(..) )
 import Darcs.Util.Exception ( clarifyErrors )
+import Darcs.Util.Path
+    ( SubPath, toFilePath, getUniquePathName, floatPath )
+import Darcs.Util.Printer ( putDocLn, text, (<>), (<+>) )
 import Darcs.Util.Prompt ( PromptConfig(..), promptChar )
+import Darcs.Util.Text ( pathlist )
 
-announceFiles :: Maybe [SubPath] -> String -> IO ()
-announceFiles Nothing _ = return ()
-announceFiles (Just files) message = putStrLn $ message ++ " " ++
-    unwords (map show files) ++ ":\n"
+announceFiles :: Verbosity -> Maybe [SubPath] -> String -> IO ()
+announceFiles Quiet _ _ = return ()
+announceFiles _ (Just subpaths) message = putDocLn $
+    text message <> text ":" <+> pathlist (map toFilePath subpaths)
+announceFiles _ _ _ = return ()
 
 testTentativeAndMaybeExit :: (RepoPatch p, ApplyState p ~ Tree)
-                          => Repository p wR wU wT
+                          => Repository rt p wR wU wT
                           -> Verbosity
                           -> TestChanges
                           -> SetScriptsExecutable
@@ -74,29 +78,34 @@ testTentativeAndMaybeExit repo verb test sse interactive failMessage confirmMsg
         yn <- promptChar (PromptConfig prompt "yn" [] (Just 'n') [])
         unless (yn == 'y') doExit
 
-filterExistingFiles :: (RepoPatch p, ApplyState p ~ Tree)
-                    => Repository p wR wU wT
-                    -> LookForAdds
+-- | Given a repository and two common command options, classify the given list
+-- of subpaths according to whether they exist in the pristine or working tree.
+-- Paths which are neither in working nor pristine are reported and dropped.
+-- The result is a pair of path lists: those that exist only in the working tree,
+-- and those that exist in pristine or working.
+filterExistingPaths :: (RepoPatch p, ApplyState p ~ Tree)
+                    => Repository rt p wR wU wT
+                    -> Verbosity
+                    -> UseIndex
+                    -> ScanKnown
                     -> [SubPath]
-                    -> IO [SubPath]
-filterExistingFiles repo lfa files = do
+                    -> IO ([SubPath],[SubPath])
+filterExistingPaths repo verb useidx scan paths = do
       pristine <- readRecorded repo
-      -- TODO this is slightly inefficient, since we should really somehow
-      -- extract the unrecorded state as a side-effect of unrecordedChanges
-      index <- readUnrecorded repo $ Just files
-      nonboring <- restrictBoring index
-      working <- applyTreeFilter nonboring `fmap` readPlainTree "."
-      let paths = map toFilePath files
-          check = virtualTreeIO $ mapM (exists . floatPath) paths
-      (in_working, _) <- check working
+      working <- readUnrecordedFiltered repo useidx scan (Just paths)
+      let filepaths = map toFilePath paths
+          check = virtualTreeIO $ mapM (exists . floatPath) filepaths
       (in_pristine, _) <- check pristine
-      mapM_ maybe_warn $ zip3 paths in_working in_pristine
-      return [ path | (path, True) <- zip files (zipWith (||) in_working in_pristine) ]
-    where maybe_warn (file, False, False) =
-              putStrLn $ "WARNING: File '"++file++"' does not exist!"
-          maybe_warn (file, True, False) | lfa == YesLookForAdds =
-              putStrLn $ "WARNING: File '" ++ file ++ "' not in repository!"
-          maybe_warn _ = return ()
+      (in_working, _) <- check working
+      let paths_with_info       = zip3 paths in_pristine in_working
+          paths_in_neither      = [ p | (p,False,False) <- paths_with_info ]
+          paths_only_in_working = [ p | (p,False,True) <- paths_with_info ]
+          paths_in_either       = [ p | (p,inp,inw) <- paths_with_info, inp || inw ]
+          or_not_added          = if scan == ScanKnown then " or not added " else " "
+      unless (verb == Quiet || null paths_in_neither) $ putDocLn $
+        "Ignoring non-existing" <> or_not_added <> "paths:" <+>
+        pathlist (map toFilePath paths_in_neither)
+      return (paths_only_in_working, paths_in_either)
 
 getUniqueRepositoryName :: Bool -> FilePath -> IO FilePath
 getUniqueRepositoryName talkative name = getUniquePathName talkative buildMsg buildName
