@@ -104,6 +104,7 @@ module Darcs.UI.Options.All
     , UseIndex (..) -- re-export
     , ScanKnown (..) -- re-export
     , diffing
+    , IncludeBoring (..)
     , includeBoring
     , allowProblematicFilenames
     , allowCaseDifferingFilenames
@@ -234,6 +235,9 @@ module Darcs.UI.Options.All
     , optimizePatchIndex
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( (^) )
 import Data.Char ( isDigit )
 import Data.List ( intercalate )
@@ -265,6 +269,8 @@ import Darcs.Repository.Flags
     , WantGuiPause (..)
     , WithPatchIndex (..)
     , WithWorkingDir (..)
+    , PatchFormat (..)
+    , IncludeBoring (..)
     )
 
 import qualified Darcs.UI.Options.Flags as F ( DarcsFlag(..) )
@@ -663,17 +669,17 @@ useIndex = (imap . cps) (Iso fw bw) ignoreTimes where
 
 scanKnown :: PrimDarcsOption ScanKnown
 scanKnown = imap (Iso fw bw) $ lookforadds ^ includeBoring where
-  fw k ScanKnown = k NoLookForAdds False
-  fw k ScanAll = k YesLookForAdds False
-  fw k ScanBoring = k YesLookForAdds True
+  fw k ScanKnown = k NoLookForAdds NoIncludeBoring
+  fw k ScanAll = k YesLookForAdds NoIncludeBoring
+  fw k ScanBoring = k YesLookForAdds YesIncludeBoring
   bw k NoLookForAdds _ = k ScanKnown
-  bw k YesLookForAdds False = k ScanAll
-  bw k YesLookForAdds True = k ScanBoring
+  bw k YesLookForAdds NoIncludeBoring = k ScanAll
+  bw k YesLookForAdds YesIncludeBoring = k ScanBoring
 
-includeBoring :: PrimDarcsOption Bool
-includeBoring = withDefault False
-  [ RawNoArg [] ["boring"] F.Boring True "don't skip boring files"
-  , RawNoArg [] ["no-boring"] F.SkipBoring False "skip boring files" ]
+includeBoring :: PrimDarcsOption IncludeBoring
+includeBoring = withDefault NoIncludeBoring
+  [ RawNoArg [] ["boring"] F.Boring YesIncludeBoring "don't skip boring files"
+  , RawNoArg [] ["no-boring"] F.SkipBoring NoIncludeBoring "skip boring files" ]
 
 allowProblematicFilenames :: DarcsOption a (Bool -> Bool -> a)
 allowProblematicFilenames = allowCaseDifferingFilenames ^ allowWindowsReservedFilenames
@@ -1094,7 +1100,10 @@ __humanReadable val = RawNoArg [] ["human-readable"] F.HumanReadable val "give h
 
 -- | See above.
 machineReadable :: PrimDarcsOption Bool
-machineReadable = singleNoArg [] ["machine-readable"] F.MachineReadable "give machine-readable output"
+machineReadable = withDefault False [__machineReadable True]
+
+__machineReadable :: RawDarcsOption
+__machineReadable val = RawNoArg [] ["machine-readable"] F.MachineReadable val "give machine-readable output"
 
 -- ** clone
 
@@ -1127,14 +1136,12 @@ hashed = deprecated
   , "Use --darcs-1 to get the effect that --hashed had previously." ] $
   [ RawNoArg [] ["hashed"] F.Hashed () "deprecated, use --darcs-1 instead" ]
 
-data PatchFormat = PatchFormat1 | PatchFormat2 deriving (Eq, Show)
-
 patchFormat :: PrimDarcsOption PatchFormat
 patchFormat = withDefault PatchFormat2
   [ RawNoArg [] ["darcs-2"] F.UseFormat2 PatchFormat2
     "Standard darcs patch format"
   , RawNoArg [] ["darcs-1"] F.UseFormat1 PatchFormat1
-    "Older patch format (for compatibility)"]
+    "Older patch format (for compatibility)" ]
 
 -- ** dist
 
@@ -1148,13 +1155,14 @@ distzip = singleNoArg [] ["zip"] F.DistZip "generate zip archive instead of gzip
 
 -- ** log
 
-data ChangesFormat = HumanReadable | GenContext | GenXml | NumberPatches | CountPatches deriving (Eq, Show)
+data ChangesFormat = HumanReadable | MachineReadable | GenContext | GenXml | NumberPatches | CountPatches deriving (Eq, Show)
 
 changesFormat :: PrimDarcsOption (Maybe ChangesFormat)
 changesFormat = withDefault Nothing
   [ RawNoArg [] ["context"] F.GenContext (Just GenContext) "give output suitable for get --context"
   , __xmloutput (Just GenXml)
   , __humanReadable (Just HumanReadable)
+  , __machineReadable (Just MachineReadable)
   , RawNoArg [] ["number"] F.NumberPatches (Just NumberPatches) "number the changes"
   , RawNoArg [] ["count"] F.Count (Just CountPatches) "output count of changes" ]
 
