@@ -1,4 +1,8 @@
 module Darcs.UI.Options.Iso where
+
+import Prelude ()
+import Darcs.Prelude
+
 -- * Isomorphisms
 
 -- | Lightweight type ismomorphisms (a.k.a. invertible functions). If
