@@ -3,8 +3,8 @@ module Darcs.Util.Environment
       maybeGetEnv
     ) where
 
-
-import Prelude hiding ( catch )
+import Prelude ()
+import Darcs.Prelude
 
 import System.Environment ( getEnv )
 
