@@ -313,10 +313,9 @@ instance FilePathLike SubPath where
 
 class CharLike c where
   toChar :: c -> Char
-  fromChar :: Char -> c
+
 instance CharLike Char where
   toChar = id
-  fromChar = id
 
 instance CharLike c => FilePathLike [c] where
   toFilePath = map toChar
