@@ -120,9 +120,11 @@ static const char *perform()
 
 const char *curl_request_url(const char *url,
                              const char *filename,
-                             int cache_time)
+                             int cache_time,
+                             int* errorCode)
 {
   int error;
+  *errorCode = -1;
 
   if (init_done == 0)
     {
@@ -172,6 +174,12 @@ const char *curl_request_url(const char *url,
       return error_strings[RESULT_FILE_OPEN_FAIL];
     }
 
+  error = set_time_out(easy, errorCode);
+  if (error != CURLE_OK ){
+    *errorCode = error;
+    return curl_easy_strerror(error);
+  }
+
   error = curl_easy_setopt(easy, CURLOPT_PRIVATE, url_data);
   if (error != CURLE_OK)
     return curl_easy_strerror(error);
@@ -349,7 +357,7 @@ int set_time_out(CURL *handle, int* errorCode)
       *errorCode = 90 ;
   }
 
-  error = curl_easy_setopt(handle, CURLOPT_TIMEOUT, time_out);
+  error = curl_easy_setopt(handle, CURLOPT_CONNECTTIMEOUT, time_out);
 
   return error;
 }
