@@ -1,29 +1,29 @@
 #!/usr/bin/env bash
 . ./lib
 
+
+# issue701
 rm -rf temp0 temp1 temp2
 
 # step 1
-mkdir temp0
+darcs init temp0 --darcs-2
 cd temp0
-darcs init --darcs-2
 echo m1 > foo
-darcs add foo
-darcs record -a -m m1 -A moi --ignore-times
+darcs record -lam m1
 cd ..
 
 # step 2
-darcs get temp0 temp1
+darcs clone temp0 temp1
 cd temp1
 echo a1 > foo
-darcs record foo -a -m a1 -A moi --ignore-times
+darcs record foo -a -m a1
 cd ..
 
 
 # step 3
 cd temp0
 echo m2 > foo
-darcs record -a -m m2 -A moi --ignore-times
+darcs record -a -m m2
 cd ..
 
 
@@ -31,26 +31,26 @@ cd ..
 cd temp1
 darcs pull -a
 echo m2-a1 > foo
-darcs record -a -m 'Fix conflict m2-a1' -A moi --ignore-times
+darcs record -a -m 'Fix conflict m2-a1'
 cd ..
 
 #step 5
 cd temp0
 echo m3 > foo
-darcs record -a -m m3 -A moi --ignore-times
+darcs record -a -m m3
 cd ..
 
 #step 6
-darcs get temp0 temp2
+darcs clone temp0 temp2
 cd temp2
 echo b1 > foo
-darcs record -a -m b1 -A moi --ignore-times
+darcs record -a -m b1
 cd ..
 
 #step 7
 cd temp0
 echo m4 > foo
-darcs record -a -m m4 -A moi --ignore-times
+darcs record -a -m m4
 cd ..
 
 #step 8
