@@ -1,13 +1,15 @@
 #!/usr/bin/env bash
 . lib
 
+#pragma repo-format darcs-1,darcs-2
+
 rm -rf temp1 temp2
 
-if grep old-fashioned .darcs/defaults; then
-format=old-fashioned-inventory
+if grep darcs-1 .darcs/defaults; then
+format=hashed
 elif grep darcs-2 .darcs/defaults; then
 format=darcs-2
-else format=hashed; fi
+else format=ERROR; fi
 
 mkdir temp2
 cd temp2
